<?php

namespace Teknisa\OPL\Factories;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ContabilizacaoFactory {

    protected $connection;
    protected $entityManager;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        // $this->entityManager = $entityManager;
    }


    /********************************************************************
     *                                                                  *
     *                     Funções de Contabilização                    *
     *                                                                  *
     ********************************************************************/

    /*
     * Executa procedure para contabilização de um lançamento de recebimento [CTB_REC_LANC].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbRecLanc($cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $dtajusterec, $cdajuctrec, $idtipo) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_REC_LANC');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPARCELAREC', Param::PARAM_INPUT, $nrparcelarec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQVENREC', Param::PARAM_INPUT, $nrseqvenrec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTAJUSTEREC', Param::PARAM_INPUT, $dtajusterec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDAJUCTREC', Param::PARAM_INPUT, $cdajuctrec, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para a contabilização de uma baixa a receber [CTB_REC_BAIXA].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbRecBaixa($cdempresa, $cdfilial, $cdfiliproc, $nrbaixa) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_REC_BAIXA');
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIPROC', Param::PARAM_INPUT, $cdfiliproc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRBAIXA', Param::PARAM_INPUT, $nrbaixa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para a contabilização de um lançamento de controle bancário [CTB_BCO_LANC].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbBcoLanc($cdreducont, $nrmovi, $idtipo) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_BCO_LANC');
        $procedure->addParam(new Param('P_CDREDUCONT', Param::PARAM_INPUT, $cdreducont, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRMOVI', Param::PARAM_INPUT, $nrmovi, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para a contabilização de uma nota fiscal de entrada [CTB_DFE_CONTAB].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbDfeContab($cdfilial, $nrlanctonf) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_DFE_CONTAB');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();
        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para a contabilização de uma nota fiscal de saída [CTB_DFS_CONTAB].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbDfsContab($cdfilial, $nrlanctonf) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_DFS_CONTAB');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure contabil pagar [CTB_PAG_LANC].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbPagLanc($cdfilial, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $dtajustepag, $cdajuctpag, $idtipo) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_PAG_LANC');
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPARCELAPAG', Param::PARAM_INPUT, $nrparcelapag, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQVENPAG', Param::PARAM_INPUT, $nrseqvenpag, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTAJUSTEPAG', Param::PARAM_INPUT, $dtajustepag, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDAJUCTPAG', Param::PARAM_INPUT, $cdajuctpag, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para a contabilização de uma baixa a pagar [CTB_BAIXA_PAG].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbBaixaPag($cdempresa, $cdfilial, $nrbaixa) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_BAIXA_PAG');
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRBAIXA', Param::PARAM_INPUT, $nrbaixa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }

    /*
     * Executa procedure para contabilização de transferência bancária [CTB_BCO_TRANSF].
     * Retorna o número do lançamento.
     */
    public function callProcedureCtbBcoTransf($cdempresa, $nrtranbanc, $dttranbanc, $cdfiliproc) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CTB_BCO_TRANSF');
        $procedure->addParam(new Param('P_NRTRANBANC', Param::PARAM_INPUT, $nrtranbanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATA', Param::PARAM_INPUT, $dttranbanc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $cdfiliproc, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 12));
        $output = $procedure->execute();

        // Verifica se o número de lançamento foi retornado corretamente.
        $nrlancctb = !empty($output['P_NRLANCCTB']) ? $output['P_NRLANCCTB'] : null;
        return $nrlancctb;
    }


    /********************************************************************
     *                                                                  *
     *                 Funções de Manipulação dos Dados                 *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para retornar os dados das partidas de um lançamento, dados os parâmetros.
     */
    public function getPartidasLancamento($params) {
        // Query a ser executada.
        $sql = "SELECT L.CDEMPRESA, L.NRLANCCONT AS NRLANCCTBTEMP, L.NRSEQLANC AS NRSEQCTBTEMP, C.CDCONTCTBL, C.NMCONTCTBL, TO_CHAR(L.DTLANCCONT, 'DD/MM/YYYY') AS DTLANCCTBTEMP, DECODE(L.VRLANCAMENT, ABS(L.VRLANCAMENT), 'CR', 'DB') IDDEBCRE, ABS(L.VRLANCAMENT) AS VRLANCCTBTEMP, E.NMEMPRESA, L.CDDOCUCONT, L.DSHISTLANC AS DSHISTCTB, C.IDTIPOCOME AS IDLANCCTBCOME, L.CDCONTCOME AS CDLANCCTBCOME, C.CDCONTREDU, L.CDHISTPADR, L.NRORG FROM LANCCONT L, CONTCTBL C, EMPRESA E WHERE L.CDCONTCTBL = C.CDCONTCTBL AND L.NRORG = C.NRORG AND L.CDEMPRESA = E.CDEMPRESA AND L.NRORG = E.NRORG AND L.NRLANCCONT = :NRLANCCONT AND L.CDEMPRESA = :CDEMPRESA AND L.NRORG = :NRORG ORDER BY L.NRLANCCONT, L.NRSEQLANC";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os.
        return $statement->fetchAll();
    }

    /*
     * Função para retornar os dados dos rateios de um lançamento, dados os parâmetros.
     */
    public function getRateiosLancamento($params) {
        // Query a ser executada.
        $sql = "SELECT L.CDEMPRESA, L.NRLANCCONT AS NRLANCCTBTEMP, L.NRSEQLANC AS NRSEQCTBTEMP, L.CDFILIAL, F.NMFILIAL, L.CDCENTCUST, C.NMCENTCUST, L.DTCOMPCUSTO AS DTCOMPCTBTEMP, DECODE(L.VRLANCCUST, ABS(L.VRLANCCUST), 'CR', 'DB') AS IDDEBCRE, ABS(L.VRLANCCUST) AS VRRATECTBTEMP FROM LANCCCUSTO L, FILIAL F, CENTCUST C WHERE L.CDEMPRESA = :CDEMPRESA AND L.NRLANCCONT = :NRLANCCONT AND L.NRORG = :NRORG AND L.CDFILIAL = F.CDFILIAL AND L.NRORG = F.NRORG AND L.CDCENTCUST = C.CDCENTCUST AND L.NRORG = C.NRORG ORDER BY L.CDEMPRESA, L.NRLANCCONT, L.NRSEQLANC, L.CDFILIAL, L.CDCENTCUST";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch em todos os dados, retornando-os
        return $statement->fetchAll();
    }

    /*
     * Função para buscar os parâmetros gerais bancários na tabela GERAL_BCO.
     */
    public function getParametrosBancarios($nrorg) {
        $params = array(
            "NRORG" => $nrorg
        );

        $sql = "SELECT * FROM GERAL_BCO WHERE CDGERALBCO = '1' AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetch(\PDO::FETCH_ASSOC);
    }

    /*
     * Função para buscar dados na tabela temporária de erros.
     * As procedures de contabilização inserem dados nesta tabela automaticamente
     * ao encontrar quaisquer problemas de contabilização.
     * Caso não haja dados na tabela, significa que não houve erros de
     * contabilização no processo.
     */
    public function getErrosTemporarios($params) {
        $sql = "SELECT NRORG, CDEMPRESA, NRLANCCTBTEMP, CDERROTEMP, NRSEQCTBTEMP FROM ERROCTBTEMP WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCTBTEMP = :NRLANCCTBTEMP AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /*
     * Função para excluir os dados da tabela temporária de erros para um
     * dado lançamento.
     */
    public function deleteErrosTemporarios($params) {
        $sql = "DELETE FROM ERROCTBTEMP WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCTBTEMP = :NRLANCCTBTEMP AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /*
     * Função para buscar dados na tabela temporária de lançamentos.
     * As procedures de contabilização inserem os lançamentos do processo em questão
     * nesta tabela automaticamente.
     */
    public function getPartidasTemporarias($params) {
        $sql = "SELECT L.NRORG, L.CDEMPRESA,  L.NRLANCCTBTEMP, L.NRSEQCTBTEMP, L.CDCONTCTBL, C.CDCONTREDU, C.NMCONTCTBL, L.CDHISTPADR, L.DSHISTCTB, TO_CHAR(L.DTLANCCTBTEMP, 'DD/MM/YYYY') DTLANCCTBTEMP, ABS(L.VRLANCCTBTEMP) VRLANCCTBTEMP, DECODE(L.VRLANCCTBTEMP, ABS(L.VRLANCCTBTEMP), 'CR', 'DB') IDDEBCRE, L.IDLANCCTBCOME, L.CDLANCCTBCOME, E.NMEMPRESA, L.CDDOCUTEMP CDDOCUCONT FROM LANCCTBTEMP L, CONTCTBL C, EMPRESA E WHERE L.CDCONTCTBL = C.CDCONTCTBL(+) AND L.NRORG = C.NRORG(+) AND L.CDEMPRESA  = E.CDEMPRESA AND L.NRORG  = E.NRORG AND L.CDEMPRESA  = :CDEMPRESA AND L.NRLANCCTBTEMP = :NRLANCCTBTEMP AND L.NRORG = :NRORG ORDER BY NRSEQCTBTEMP";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /*
     * Função para buscar excluir os dados da tabela temporária de lançamentos.
     */
    public function deletePartidasTemporarias($params) {
        $sql = "DELETE FROM LANCCTBTEMP WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCTBTEMP = :NRLANCCTBTEMP AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /*
     * Função para buscar dados na tabela temporária de rateios de contas.
     * As procedures de contabilização inserem os rateios automaticamente dos
     * lançamentos do processo nesta tabela automaticamente, caso a parametrização
     * permita o rateio.
     */
    public function getRateiosTemporarios($params) {
        $sql = "SELECT R.NRORG, R.CDFILIAL, F.NMFILIAL, R.CDCENTCUST, C.NMCENTCUST, TO_CHAR(R.DTCOMPCTBTEMP, 'DD/MM/YYYY') DTCOMPCTBTEMP, ABS(R.VRRATECTBTEMP) VRRATECTBTEMP, DECODE(R.VRRATECTBTEMP, ABS(R.VRRATECTBTEMP), 'CR', 'DB') IDDEBCRE, R.NRSEQCTBTEMP, R.NRLANCCTBTEMP, R.CDEMPRESA, R.NRSEQRATTEMP, R.NRORG FROM RATECTBTEMP R, FILIAL F, CENTCUST C, LANCCTBTEMP L WHERE R.NRLANCCTBTEMP = :NRLANCCTBTEMP AND R.CDEMPRESA = :CDEMPRESA AND R.NRORG = :NRORG AND R.CDFILIAL = F.CDFILIAL(+) AND R.NRORG = F.NRORG(+) AND R.CDCENTCUST = C.CDCENTCUST(+) AND R.NRORG = C.NRORG(+) AND R.NRLANCCTBTEMP = L.NRLANCCTBTEMP AND R.NRSEQCTBTEMP = L.NRSEQCTBTEMP ORDER BY R.CDFILIAL, R.CDCENTCUST";

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        return $std->fetchAll();
    }

    /*
     * Função para buscar excluir os dados da tabela temporária de rateios.
     */
    public function deleteRateiosTemporarios($params) {
        $sql = "DELETE FROM RATECTBTEMP WHERE CDEMPRESA = :CDEMPRESA AND NRLANCCTBTEMP = :NRLANCCTBTEMP AND NRORG = :NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}