<?php

namespace Teknisa\OPL\Services;

use Teknisa\OPL\Services\Dfe11000NfEntradaService;
use Teknisa\OPL\Services\Dfs11000NfSaidaService;
use Teknisa\OPL\Services\ReplicacaoDocumentosService;

use Doctrine\ORM\EntityManager;
use Indeterminado\Entities\Aliqtempprod; // TODO REMOVER
use Estoque\Entities\Lanctoestoq;
use Financeiro\Entities\Contareceber;
use Fiscal\Entities\Contapagar;
use Fiscal\Entities\Dadosadic;
use Fiscal\Entities\Impostonf;
use Fiscal\Entities\Itemproimpos;
use Fiscal\Entities\Trabitproimpos;
use Fiscal\Entities\Trabimposnftot;
use Fiscal\Entities\Notafiscal;
use Fiscal\Entities\Incitemp;
use Fiscal\Entities\IntgNotafiscal;
use Fiscal\Entities\Rateionf;
use Geral\Entities\Encargo;
use Geral\Entities\Imposto;
use Geral\Entities\Itemencargo;
use Geral\Entities\Itemproduto;
use Geral\Entities\Produto;
use Geral\Entities\Prodfili;
use Geral\Entities\Servico;
use Geral\Entities\Empresa;
use Geral\Entities\Endeforn;
use Geral\Entities\Endeclie;
use Geral\Entities\Endefili;
use Geral\Entities\Tipoctpagar;
use Teknisa\Libs\Util\DateUtil;
use Qualificacao\Entities\Movocorr;
use Teknisa\OPL\Services\ValidacoesEstoqueService;
use Teknisa\OPL\Factories\NotafiscalLancamentoFactory;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\OPL\Services\AprovacaoLancamentosService;
use Zeedhi\Framework\DTO\Response;

class LancamentoNotafiscalService {
    const VALUE = 'VALUE';
    const VALUE_COLUMN = 'VALUE_COLUMN';
    const ESTOQUE_SEPARADOR = ' ';

    const NOVO_CODIGO_LANC_ESTOQUE  = 'LANCTOEST';
    const PROCEDURE_DELETA_NF       = 'DELETA_NF';
    const PROCEDURE_GERA_PRODUTO    = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_SAIDA = 'GERA_NOTAFISCAL_FECHAMENTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO_ENTRADA = 'GERA_NOTAFISCAL_CORPO_ENTRADA';
    const PROCEDURE_GERA_NOTA_CORPO         = 'GERA_NOTAFISCAL_CORPO';
    const PROCEDURE_CONSULTA_LOTE           = 'CONSULTA_LOTE';

    protected $entityManager;
    protected $novoCodigo;
    protected $validacoesEstoqueService;
    protected $environment;
    protected $LancamentoNotafiscalFactory;
    protected $aprovacaoLancamentosService;
    protected $erros = [];

    private $errosAvisos = array();

    public function __construct(EntityManager $entityManager, NewCode $novoCodigo, ValidacoesEstoqueService $validacoesEstoqueService, $environment, AprovacaoLancamentosService $aprovacaoLancamentosService = null, $manifestacaoService){
        $this->novoCodigo = $novoCodigo;
        $this->entityManager = $entityManager;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->environment = $environment;
        date_default_timezone_set('America/Sao_Paulo');
        $this->lancamentoFactory = new NotafiscalLancamentoFactory($entityManager);
        $this->aprovacaoLancamentosService = $aprovacaoLancamentosService;
        $this->manifestacaoService = $manifestacaoService;
    }

    public function buscaProdutos($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $itemNotaFisc = $this->lancamentoFactory->getItemNotaFisc($filial, $lancamento);
        $produtos = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        $arrayProdutos = $this->criaArrayProdutos($produtos, $itemNotaFisc);
        return $arrayProdutos;
    }
    
    public function buscaProdFili($filial, $arrayProdutos) {
        $organizacao = $this->environment->getNrOrgTrab();
        $arrayProdFili = [];
        foreach($arrayProdutos as $produto){
            $cdproduto = $produto['CDPRODUTO'];
            $dadosProduto = Prodfili::findByCdfilialAndCdprodutoAndNrorg($filial, $cdproduto, $organizacao);
            array_push($arrayProdFili, $dadosProduto);
        }
        return($arrayProdFili);
    }
    
    public function buscaProdEsto($arrayProdutos) {
        $organizacao = $this->environment->getNrOrgTrab();
        $arrayProdEsto = [];
        foreach($arrayProdutos as $produto){
            $cdproduto = $produto['CDPRODUTO'];
            $dadosProduto = Produto::findByCdprodutoAndNrorg($cdproduto, $organizacao);
            array_push($arrayProdEsto, $dadosProduto);
        }
        return($arrayProdEsto);
    }

    public function criaArrayProdutos(array $itemProdutos, $itemNotaFisc) {
        
        $organizacao    = $this->environment->getNrOrgTrab();
        $arrayProdutos  = array();
        $quantidadeProdutosOrigem = array();
        if (!is_array($itemProdutos))
            return $arrayProdutos;
            
        if($itemNotaFisc) {
            
            $produtosOrigem = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($itemNotaFisc["CDFILIALIT"], $itemNotaFisc["NRLANCTONFIT"], $organizacao);
            
            foreach($produtosOrigem as $prodOrigem) {
                if(!isset($quantidadeProdutosOrigem[$prodOrigem->getCdproduto()]) ) {
                    $quantidadeProdutosOrigem[$prodOrigem->getCdproduto()] = $prodOrigem->getQtitemprod();
                }
                else {
                    $quantidadeProdutosOrigem[$prodOrigem->getCdproduto()] += $prodOrigem->getQtitemprod();
                }
            }
            
        }
   
        /** @var Itemproduto $item */
        foreach ($itemProdutos as $key => $item) {
            if (!$item instanceof Itemproduto)
                throw new Exception('Produto não encontrado.');
            
            $produto    = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $organizacao);
            $prodfili   = Prodfili::findOneByCdfilialAndCdprodutoAndNrorg($item->getCdfilimovi(), $item->getCdproduto(), $organizacao);
            if($item->getCdservfili()) {
                $servico = $this->lancamentoFactory->buscaServico($item->getCdfilimovi(), $item->getCdservfili());
    
                if(isset($servico[0])) {
                    $arrayProdutos[$key]["NMPRODUTO"] = $servico[0]["NMSERVICO"];
                }
                else {
                    $arrayProdutos[$key]['NMPRODUTO'] = $produto->getNmproduto();
                }
            }
            else {
                $arrayProdutos[$key]['NMPRODUTO'] = $produto->getNmproduto();
            }
      
            $arrayProdutos[$key]['CDFILIAL']     = $item->getCdfilial();
            $arrayProdutos[$key]['NRLANCTONF']   = $item->getNrlanctonf();
            $arrayProdutos[$key]['CDPRODUTO']    = $item->getCdproduto();
            $arrayProdutos[$key]['NRSEQITPROD']  = $item->getNrseqitprod();
            $arrayProdutos[$key]['CDPRODEMIS']   = $item->getCdprodemis();
            $arrayProdutos[$key]['DSPRODEMIS']   = $item->getDsprodemis();
            $arrayProdutos[$key]['QTITEMPROD']   = $item->getQtitemprod();
            $arrayProdutos[$key]['VRUNITITEM']   = $item->getVrunititem();
            $arrayProdutos[$key]['VRTOTAITEM']   = $this->truncFloat($item->getVrtotaitem(), 2);
            $arrayProdutos[$key]['VRDESCITEM']   = $item->getVrdescitem();
            $arrayProdutos[$key]['VRCONTITEM']   = $item->getVrcontitem();
            $arrayProdutos[$key]['CDSITUTRIBPR'] = $item->getCdsitutribpr();
            $arrayProdutos[$key]['CDFISOPEITEM'] = $item->getCdfisopeitem();
            $arrayProdutos[$key]['CDFISCOPERSN'] = $item->getCdfiscopersn();
            $arrayProdutos[$key]['NRPEDPROD']    = $item->getNrpedprod();
            $arrayProdutos[$key]['NRSEQPEDPROD'] = $item->getNrseqpedprod();
            $arrayProdutos[$key]['QTVOLUITEM']   = $item->getQtvoluitem();
            $arrayProdutos[$key]['CDSERVFATU']   = $item->getCdservfatu();
            $arrayProdutos[$key]['CDSERVFILI']   = $item->getCdservfili();
            $arrayProdutos[$key]['CDALMOXARIFE'] = $item->getCdalmoxarife();
            $arrayProdutos[$key]['CDLOCALESTOQ'] = $item->getCdlocalestoq();
            $arrayProdutos[$key]['NRLOTEESTQ']   = $item->getNrloteestq();
            $arrayProdutos[$key]['NRSUBLOTE']    = $item->getNrsublote();
            $arrayProdutos[$key]['NRAUTOFORN']   = $item->getNrautoforn();
            $arrayProdutos[$key]['CDMARCA']      = $item->getCdmarca();
            $params = array(
                ':P_CDFILIAL' => $item->getCdfilial(),
                ':P_NRLANCTONF' => $item->getNrlanctonf(),
                ':P_NRSEQITPROD' => $item->getNrseqitprod(),
                ':P_NRORG' =>$organizacao
            ); //Não tem getdsInfadicprod na instância da classe "Geral\Entities\Itemproduto".
            
            $dsInfadicprod = $this->lancamentoFactory->getdsInfadicprod($params);
            $arrayProdutos[$key]['DSINFADICPROD'] = $dsInfadicprod;

            $arrayProdutos[$key]['CDARVPROD']   = $produto->getCdarvprod();
            $arrayProdutos[$key]['SGUNIDADE']   = $produto->getSgunidade();
            $arrayProdutos[$key]['VRFATOCONV']  = $produto->getVrfatoconv();
            $arrayProdutos[$key]['CDPRODESTO']  = $produto->getCdprodesto();
            
            $errosAvisos = $this->lancamentoFactory->getErrosAvisosItem($item, $organizacao);
            
            $arrayProdutos[$key]['IDERROSITEM'] = $errosAvisos['IDERROSITEM'];
            $arrayProdutos[$key]['IDAVISOSITEM'] = $errosAvisos['IDAVISOSITEM'];
            
            // if($itemNotaFisc) {
            //     $arrayProdutos[$key]["QTITEMORIG"] = $quantidadeProdutosOrigem[$item->getCdproduto()] ? $quantidadeProdutosOrigem[$item->getCdproduto()] : 0;
            // }
            // else {
            //     $arrayProdutos[$key]["QTITEMORIG"] = 0;
            // }
            
            if(is_null($prodfili)) {
                $arrayProdutos[$key]['IDCNTRESTOQ']   = 'S';
            }
            else {
                $arrayProdutos[$key]['IDCNTRESTOQ'] = $prodfili->getIdcntrestoq();
            }
        }
        
        return $arrayProdutos;
    }
    
    private function truncFloat($num, $precision = 2) {
        return round($num, $precision);
    }


    public function buscaImpostosProdutos($filial, $lancamento, $entrada_saida, $statusNota) {
        
        $organizacao = $this->environment->getNrOrgTrab();

        if ($entrada_saida === Notafiscal::NOTA_FISCAL_ENTRADA && $statusNota === Notafiscal::EM_DIGITACAO) {
            return $this->buscaImpostosProdutosTemp($filial, $lancamento, $organizacao);
        }

        $impostos = $this->entityManager->getRepository('Teknisa\Model\Fiscal\Entities\Itemproimpos')->findBy(array('cdfilial' => $filial, 'nrlanctonf' => $lancamento));
        
        $arrayImpostosProdutos = array();

        /** @var Itemproimpos $itemproimpos */
        foreach ($impostos as $key => $itemproimpos) {
            /** @var Imposto $imposto */

            $imposto = Imposto::findOneByCdimpostoAndNrorg($itemproimpos->getCdimposto(), $organizacao);
            $iddescrfonte = !!$imposto->getIddescrfonte() ? $imposto->getIddescrfonte() : 'R';

            $arrayImpostosProdutos[$key]['CDFILIAL']        = $itemproimpos->getCdfilial();
            $arrayImpostosProdutos[$key]['NRLANCTONF']      = $itemproimpos->getNrlanctonf();
            $arrayImpostosProdutos[$key]['NRSEQITPROD']     = $itemproimpos->getNrseqitprod();
            $arrayImpostosProdutos[$key]['CDIMPOSTO']       = $itemproimpos->getCdimposto();
            $arrayImpostosProdutos[$key]['VRIMPOSTOIT']     = $itemproimpos->getVrimpostoit();
            $arrayImpostosProdutos[$key]['IDDESCRFONTE']    = $iddescrfonte;
            $arrayImpostosProdutos[$key]['IDTRATIMPO']      = $imposto->getIdtratimpo();
            $arrayImpostosProdutos[$key]['NRITPROIMPOS']    = $itemproimpos->getNritproimpos();
            $arrayImpostosProdutos[$key]['VRCONTITIMP']     = $itemproimpos->getVrcontitimp();
            $arrayImpostosProdutos[$key]['VRPERREDBCIT']    = $itemproimpos->getVrperredbcit();
            $arrayImpostosProdutos[$key]['VRBASECALCIT']    = $itemproimpos->getVrbasecalcit();
            $arrayImpostosProdutos[$key]['VRPARDEDIT']      = $itemproimpos->getVrpardedit();
            $arrayImpostosProdutos[$key]['CDFISCALIT']      = $itemproimpos->getCdfiscalit();
            $arrayImpostosProdutos[$key]['CDSITUTRIBIT']    = $itemproimpos->getCdsitutribit();
            $arrayImpostosProdutos[$key]['CDSITUTRIBITNFE'] = $itemproimpos->getCdsitutribitnfe();
            $arrayImpostosProdutos[$key]['CDFISCOPERIT']    = $itemproimpos->getCdfiscoperit();
            $arrayImpostosProdutos[$key]['CDREGRAIT']       = $itemproimpos->getCdregrait();
            $arrayImpostosProdutos[$key]['VRALIQITEM']      = $itemproimpos->getVraliqitem(); 
            $arrayImpostosProdutos[$key]['IDMODBCITNFE']    = $itemproimpos->getIdmodbcitnfe(); 
            $arrayImpostosProdutos[$key]['IDMODBCSTITNFE']  = $itemproimpos->getIdmodbcstitnfe();
            $arrayImpostosProdutos[$key]['CDTPCREDPICOITE'] = $itemproimpos->getCdtpcredpicoite();
            $arrayImpostosProdutos[$key]['CDNATUBCALCPCIT'] = $itemproimpos->getCdnatubcalpcit();
            $arrayImpostosProdutos[$key]['CDDETALHEPROD']   = $itemproimpos->getCddetalheprod();
            $arrayImpostosProdutos[$key]['CDSIMPLESNACIONAL'] = $itemproimpos->getCdsimplesnacional();
            $arrayImpostosProdutos[$key]['IDVALORINFIT']    = $itemproimpos->getIdvalorinfit();
            $params = array(
                ':P_CDFILIAL' => $itemproimpos->getCdfilial(),
                ':P_NRLANCTONF' => $itemproimpos->getNrlanctonf(),
                ':P_NRSEQITPROD' => $itemproimpos->getNrseqitprod(),
                ':P_CDIMPOSTO'=> $itemproimpos->getCdimposto(),
                ':P_NRITPROIMPOS' => $itemproimpos->getNritproimpos(),
                ':P_NRORG' =>$organizacao
            );
            $itemproimposAll = $this->lancamentoFactory->getAllItemproimpos($params);
            $arrayImpostosProdutos[$key]['VRADICALIQFECP']  = $itemproimposAll['VRADICALIQFECP'];
            $arrayImpostosProdutos[$key]['VRPERMAJBCIT']    = $itemproimposAll['VRPERMAJBCIT'];
            $arrayImpostosProdutos[$key]['CDCEST']          = $itemproimposAll['CDCEST'];
            $arrayImpostosProdutos[$key]['VRADICFECP']      = $itemproimposAll['VRADICFECP'];
            $arrayImpostosProdutos[$key]['CDENQLEGALIPI']   = $itemproimposAll['CDENQLEGALIPI'];
            $arrayImpostosProdutos[$key]['CDBENEFFISCAL']   = $itemproimposAll['CDBENEFFISCAL'];
            $arrayImpostosProdutos[$key]['IDCONSDESONVRNF'] = $itemproimposAll['IDCONSDESONVRNF'];
            $arrayImpostosProdutos[$key]['VRIMPOSTODESON']  = $itemproimposAll['VRIMPOSTODESON'];
            $arrayImpostosProdutos[$key]['IDMOTDESONERAIT'] = $itemproimposAll['IDMOTDESONERAIT'];

            $arrayImpostosProdutos[$key]['__is_new']        = false;
            $arrayImpostosProdutos[$key]['NMIMPOSTO']       = $imposto->getNmimposto();
            $arrayImpostosProdutos[$key]['SGIMPOSTO']       = $imposto->getSgimposto();
            
            if($entrada_saida === Notafiscal::NOTA_FISCAL_ENTRADA)
                $arrayImpostosProdutos[$key]['VRALIQIMPOIT']    = $itemproimpos->getVraliqitem();
            else 
                $arrayImpostosProdutos[$key]['VRALIQITEM']    = $itemproimpos->getVraliqitem();
            
        }
        
        return $arrayImpostosProdutos;
    }

    private function buscaImpostosProdutosTemp($filial, $lancamento, $organizacao) {
        $impostos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostosProdutosTemp = array();

        /** @var Trabitproimpos $trabitproimpos */
        foreach ($impostos as $key => $trabitproimpos) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($trabitproimpos->getCdimpostoit(), $organizacao);
            $iddescrfonte = !!$imposto->getIddescrfonte() ? $imposto->getIddescrfonte() : 'R';

            $arrayImpostosProdutosTemp[$key]['CDFILIAL'] =  $trabitproimpos->getCdfilimpoit() ? $trabitproimpos->getCdfilimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['NRLANCTONF'] = $trabitproimpos->getNrlancimpoit() ? $trabitproimpos->getNrlancimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['NRSEQITPROD'] = $trabitproimpos->getNrseqimpoit() ? $trabitproimpos->getNrseqimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDIMPOSTO'] = $trabitproimpos->getCdimpostoit() ? $trabitproimpos->getCdimpostoit() : '0';
            $arrayImpostosProdutosTemp[$key]['VRALIQIMPOIT'] = $trabitproimpos->getVraliqimpoit() ? $trabitproimpos->getVraliqimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['VRIMPOSTOIT'] = $trabitproimpos->getVrimposit() ? $trabitproimpos->getVrimposit() : '0';
            $arrayImpostosProdutosTemp[$key]['IDDESCRFONTE'] = $iddescrfonte;
            $arrayImpostosProdutosTemp[$key]['CDFISCALIT'] = $trabitproimpos->getCdfiscimpoit() ? $trabitproimpos->getCdfiscimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDSITUTRIBIT']= $trabitproimpos->getCdstribimpoit() ? $trabitproimpos->getCdstribimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDSITUTRIBITNFE'] = $trabitproimpos->getCdstribitnfe() ? $trabitproimpos->getCdstribitnfe() : '0';
            $arrayImpostosProdutosTemp[$key]['CDFISCOPERIT']= $trabitproimpos->getCdfoperimpoit() ? $trabitproimpos->getCdfoperimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDREGRAIT']= $trabitproimpos->getCdregratrabit() ? $trabitproimpos->getCdregratrabit() : '0';
            $arrayImpostosProdutosTemp[$key]['IDMODBCITNFE']= $trabitproimpos->getIdmodbcitemnfe() ? $trabitproimpos->getIdmodbcitemnfe() : '0'; 
            $arrayImpostosProdutosTemp[$key]['IDMODBCSTITNFE']= $trabitproimpos->getIdmodbcstitenfe() ? $trabitproimpos->getIdmodbcstitenfe() : '0';
            $arrayImpostosProdutosTemp[$key]['IDTRIMPSTTI'] = $trabitproimpos->getIdtrimpstti() ? $trabitproimpos->getIdtrimpstti() : '0';
            $arrayImpostosProdutosTemp[$key]['NRITPROIMPOS'] = $trabitproimpos->getNrseqitpro() ? $trabitproimpos->getNrseqitpro() : '0';
            $arrayImpostosProdutosTemp[$key]['VRCONTITIMP'] = $trabitproimpos->getVrcontimpoit() ? $trabitproimpos->getVrcontimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['VRPERREDBCIT'] = $trabitproimpos->getVrredbcimpoit() ? $trabitproimpos->getVrredbcimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['VRBASECALCIT'] = $trabitproimpos->getVrbcalcimpoit() ? $trabitproimpos->getVrbcalcimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['VRPARDEDIT'] = $trabitproimpos->getVrpdedimpoit() ? $trabitproimpos->getVrpdedimpoit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDNATUBCALPCIT'] = $trabitproimpos->getCdnatubcalpcit() ? $trabitproimpos->getCdnatubcalpcit() : '0';
            $arrayImpostosProdutosTemp[$key]['CDTPCREDPICOITE'] = $trabitproimpos->getCdtpcredpicoite() ? $trabitproimpos->getCdtpcredpicoite() : '0';
            
     
            $arrayImpostosProdutosTemp[$key]['NMIMPOSTO']       = $imposto->getNmimposto();
            $arrayImpostosProdutosTemp[$key]['SGIMPOSTO']       = $imposto->getSgimposto();
        }

        return $arrayImpostosProdutosTemp;
    }
    
    public function buscaImpostos($filial, $lancamento, $entrada_saida, $statusNota) {

        $organizacao = $this->environment->getNrOrgTrab();

        if ($entrada_saida === Notafiscal::NOTA_FISCAL_ENTRADA && $statusNota === Notafiscal::EM_DIGITACAO) {
            return $this->buscaImpostosTemp($filial, $lancamento, $organizacao);
        }

        $impostos = Impostonf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostos = array();

        /** @var Impostonf $impostoNF */
        foreach ($impostos as $key => $impostoNF) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($impostoNF->getCdimposto(), $organizacao);

            $arrayImpostos[$key]['CDFILIAL'] = $impostoNF->getCdfilial();
            $arrayImpostos[$key]['NRLANCTONF'] = $impostoNF->getNrlanctonf();
            $arrayImpostos[$key]['CDIMPOSTO'] = $impostoNF->getCdimposto();
            $arrayImpostos[$key]['VRIMPOSTONF'] = $impostoNF->getVrimpostonf();
            $arrayImpostos[$key]['VRCONTABILNF'] = $impostoNF->getVrcontabilnf();
            $arrayImpostos[$key]['VRBASECALCNF'] = $impostoNF->getVrbasecalcnf();
            $arrayImpostos[$key]['VRALIQNF'] = $impostoNF->getVraliqnf();
            $arrayImpostos[$key]['CDFISCALNF'] = $impostoNF->getCdfiscalnf();
            $arrayImpostos[$key]['CDSITUTRIBNF'] = $impostoNF->getCdsitutribnf();
            $arrayImpostos[$key]['CDFISCOPERNF'] = $impostoNF->getCdfiscopernf();
            $arrayImpostos[$key]['NRSEQUMAPAPU'] = $impostoNF->getNrsequmapapu();
            
            

            $arrayImpostos[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostos[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostos;
    }

    private function buscaImpostosTemp($filial, $lancamento, $organizacao) {
        $impostos = Trabimposnftot::findByCdfilimptotAndNrlancimptotAndNrorg($filial, $lancamento, $organizacao);

        $arrayImpostos = array();

        /** @var Trabimposnftot $impostoNF */
        foreach ($impostos as $key => $impostoNF) {
            /** @var Imposto $imposto */
            $imposto = Imposto::findOneByCdimpostoAndNrorg($impostoNF->getCdimpostot(), $organizacao);

            $arrayImpostos[$key]['CDFILIAL'] = $impostoNF->getCdfilimptot();
            $arrayImpostos[$key]['NRLANCTONF'] = $impostoNF->getNrlancimptot();
            $arrayImpostos[$key]['CDIMPOSTO'] = $impostoNF->getCdimpostot();
            $arrayImpostos[$key]['VRIMPOSTONF'] = $impostoNF->getVrimpostot();
            

            $arrayImpostos[$key]['NMIMPOSTO'] = $imposto->getNmimposto();
            $arrayImpostos[$key]['SGIMPOSTO'] = $imposto->getSgimposto();
        }

        return $arrayImpostos;
    }

    public function buscaEncargos($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $encargos = Itemencargo::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        $arrayEncargos = $this->criaArrayEncargos($encargos);
        return $arrayEncargos;

        // $arrayEncargos = array();
        // /** @var Itemencargo $itemencargo */
        // foreach ($encargos as $key => $itemencargo) {
        //     $encargo = Encargo::findOneByCdencargoAndNrorg($itemencargo->getCdencargo(), $organizacao);

        //     $arrayEncargos[$key]['CDFILIAL'] = $itemencargo->getCdfilial();
        //     $arrayEncargos[$key]['NRLANCTONF'] = $itemencargo->getNrlanctonf();
        //     $arrayEncargos[$key]['CDENCARGO'] = $itemencargo->getCdencargo();
        //     $arrayEncargos[$key]['VRITEMENCA'] = $itemencargo->getVritemenca();

        //     $arrayEncargos[$key]['NMENCARGO'] = $encargo->getNmencargo();
        // }

        // return $arrayEncargos;
    }
    
    public function criaArrayEncargos(array $encargos) {

        $arrayEncargos = array();
        $organizacao = $this->environment->getNrOrgTrab();

        foreach ($encargos as $key => $itemencargo) {
            if (!$itemencargo instanceof Itemencargo)
                throw new Exception('Encargo não encontrado.');

            $encargo = Encargo::findOneByCdencargoAndNrorg($itemencargo->getCdencargo(), $organizacao);

            $arrayEncargos[$key]['CDFILIAL'] = $itemencargo->getCdfilial();
            $arrayEncargos[$key]['NRLANCTONF'] = $itemencargo->getNrlanctonf();
            $arrayEncargos[$key]['CDENCARGO'] = $itemencargo->getCdencargo();
            $arrayEncargos[$key]['VRITEMENCA'] = $itemencargo->getVritemenca();

            $arrayEncargos[$key]['NMENCARGO'] = $encargo->getNmencargo();
            $arrayEncargos[$key]['IDENCARGO'] = $encargo->getIdencargo();
        }

        return $arrayEncargos;
    }

    public function buscaVencimentos($filial, $lancamento, $entrada_saida) {
        $organizacao = $this->environment->getNrOrgTrab();
        if($entrada_saida == 'S'){
            $vencimentos = Contareceber::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        }
        else{
            $vencimentos = Contapagar::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
        }
        return $this->criaArrayVencimento($vencimentos, $entrada_saida);
    }

    public function buscaDadosAdicionais($filial, $lancamento) {
        $organizacao = $this->environment->getNrOrgTrab();
        $dadosAdicionais = Dadosadic::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $arrayDadosAdicionais = array();
        /** @var Dadosadic $dadosadic */
        foreach ($dadosAdicionais as $key => $dadosadic) {
            $arrayDadosAdicionais[$key]['CDFILIAL'] = $dadosadic->getCdfilial();
            $arrayDadosAdicionais[$key]['NRLANCTONF'] = $dadosadic->getNrlanctonf();
            $arrayDadosAdicionais[$key]['NRDADOADIC'] = $dadosadic->getNrdadoadic();
            $arrayDadosAdicionais[$key]['CDMENSAGEM'] = $dadosadic->getCdmensagem();
            $arrayDadosAdicionais[$key]['DSDADOADIC'] = $dadosadic->getDsdadoadic();
        }

        return $arrayDadosAdicionais;
    }
    
    public function buscaItensRateionf($filial, $lancamento) {
        $itensRateionf = $this->lancamentoFactory->buscaItensRateionf($filial, $lancamento);
        return $itensRateionf;
    }
    
    public function criaArrayItensRateionf(array $ItensRateionf){
        $arrayItensRateionf = array();
        $organizacao = $this->environment->getNrOrgTrab();
        
        foreach ($ItensRateionf as $key => $itemrateio) {
            if (!$itemrateio instanceof Rateionf)
                throw new Exception('Rateio não encontrado.');

            $encargo = Encargo::findOneByCdencargoAndNrorg($itemrateio->getCdencargo(), $organizacao);

            $arrayItensRateionf[$key]['CDFILIAL'] = $itemrateio->getCdfilial();
            $arrayItensRateionf[$key]['NRLANCTONF'] = $itemrateio->getNrlanctonf();
            $arrayItensRateionf[$key]['CDENCARGO'] = $itemrateio->getCdencargo();
            $arrayItensRateionf[$key]['VRITEMENCA'] = $itemrateio->getVritemenca();

            $arrayItensRateionf[$key]['NMENCARGO'] = $encargo->getNmencargo();
            $arrayItensRateionf[$key]['IDENCARGO'] = $encargo->getIdencargo();
        }
    }
    
    public function deletaRelacNotaOrigem($cdfilial, $nrlanctonf){
        $this->lancamentoFactory->deletaRelacNotaOrigem($cdfilial, $nrlanctonf);
    }
    
    public function buscaRateionf($filial, $lancamento, $gridItens) {
        $arrayItens = array();
        $arrayRateionf = array();
        if(!empty($gridItens)){
            foreach($gridItens as $item){
                array_push($arrayItens, $item['NRSEQITPROD']);
            }
            
        $stringItens = implode("_", $arrayItens);
        $arrayRateionf = $this->lancamentoFactory->buscaRateionf($filial, $lancamento, $stringItens);
        }
            
        return $arrayRateionf;
    }
    
    public function procedureDeletaNf($filial, $lancamento, $corpoItem = null){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_DELETA_NF);
            $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCORPITEM', Param::PARAM_INPUT, $corpoItem, Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function efetuaManifestacao($cdfilial, $nrlanctonf, $cdinscesta, $nracessonfe){
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDINSCESTA" => $cdinscesta,
            "P_NRACESSONFE" => $nracessonfe
        );
        
        $sql = "UPDATE NOTADESTINADA
                SET   CDFILIAL    = :P_CDFILIAL,
                      NRLANCTONF  = :P_NRLANCTONF,
                      CDINSCESTA  = :P_CDINSCESTA
                WHERE NRACESSONFE = :P_NRACESSONFE";
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getItensByCdfilialAndNrLanctoNF($cdfilial, $nrlanctonf){
        try{
            
        }catch(\Exception $e){
            throw $e;
        }
        
    }
    
    
    public function salvaInfoCompra($row) {
        $filial = $row["CDFILIAL"];
        $lancamento = $row["NRLANCTONF"];
        $notaEmp = $row["DSNOTAEMPENHO"];
        $pedido = $row["CDPEDIDO"];
        $contrato = $row["CDCONTRATO"];
        $nrorg = $row["NRORG"];

        $this->lancamentoFactory->salvaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
        
    }
    public function editaInfoCompra($row) {
        $filial = $row["CDFILIAL"];
        $lancamento = $row["NRLANCTONF"];
        $notaEmp = $row["DSNOTAEMPENHO"];
        $pedido = $row["CDPEDIDO"];
        $contrato = $row["CDCONTRATO"];
        $nrorg = $row["NRORG"];
        
        $this->lancamentoFactory->atualizaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
    }
    
    
    public function fechaNotaFiscalLiberada($dtentrsaid, $notaFiscal, $response){
        $connection = $this->entityManager->getConnection();
        try{
            $cdfilial = $notaFiscal->getCdfilial();
            $nrlanctonf = $notaFiscal->getNrlanctonf();
            
            $notaFiscal->setIdstatus('P');
            $this->atualizaStatusIntgNotaFiscal('U', $notaFiscal->getNracessonfe());
            $this->efetuaManifestacao($cdfilial, $nrlanctonf, $notaFiscal->getCdinscesta(), $notaFiscal->getNracessonfe());
            $nrlancestq = $this->lancamentoFactory->getNrLancestq($cdfilial, $nrlanctonf);
            
            if (!$nrlancestq){
                $nrlancestq = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$cdfilial, $this->environment->getNrOrgTrab());
            } else{
                $itens = $this->lancamentoFactory->getItensLancEstq($cdfilial, $nrlancestq);
                $this->lancamentoFactory->deleteCustoCmProd($itens, $cdfilial);
                $this->lancamentoFactory->alteraDtLancto($dtentrsaid, $cdfilial, $nrlancestq);
                $this->validacoesEstoqueService->atualizaEstoque($cdfilial, $nrlancestq);
            }
            $aprovAutops = $this->lancamentoFactory->getAprovAutops($cabecalho['CDTIPOOPER']) ;
            $aprovacaoAutomatica = $this->lancamentoFactory->aprovaNfXmlAutomatica();
            if($aprovAutops || $aprovacaoAutomatica) {
                $response = new Response();
                $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($this->environment->getNrOrg(), $cdfilial, $nrlanctonf, 'E', $response);
                $status = empty($errosAprovacao);
            } else{
                $status = false;
            }
            
            $infoNotaFiscal = array(
                'CABECALHO' => array(
                    'NRLANCTONF' => $nrlanctonf,
                    'VRNOTAFISC' => $notaFiscal->getVrnotafisc()
                ),
                'STATUS' => ($status) ? 'A' : 'P'
            );
            
            $connection->commit();
            return $infoNotaFiscal;
        } catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    public function fechaNotaFiscal($cabecalho, $vencimentos, $forcaFechamento){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            if($this->lancamentoFactory->verificaNotaSendoUtilizada($cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"])) {
                throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
            }
            $this->validaFecharNota($cabecalho);
            
            $filial = $cabecalho['CDFILIAL'];
            $organizacao = $this->environment->getNrOrgTrab();

            $filialSelecionada = ($cabecalho['IDENTRSAIDOP'] === Notafiscal::NOTA_FISCAL_ENTRADA) ? $cabecalho['CDFILIRECE'] : $cabecalho['CDFILIEMIT'] ;
            $numLancamento = $cabecalho['NRLANCTONF'];
            
            $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
            
            //Verifica se já existe uma NF liberada
            if($notaFiscal->getIdstatus() == 'L' && $notaFiscal->getIdentrsaidop() == 'E'){
                return $this->fechaNotaFiscalLiberada($cabecalho["DTENTRSAID"], $notaFiscal, $connection);
            }
            
            $this->validaFechamentoNotaUtilizaAf($cabecalho);

            $lancEstoqAnterior = Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
            if (!empty($lancEstoqAnterior)) {
                $numLancEstoque = $lancEstoqAnterior->getNrlancestq();
            }
            else {
                $numLancEstoque = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$filial, $organizacao);
            }

            $dadosProcessFilial = $this->validacoesEstoqueService->retornaDadosProcessFilial($filialSelecionada, $organizacao);
            
            if($cabecalho['IDENTRSAIDOP'] === Notafiscal::NOTA_FISCAL_ENTRADA) {
                $impostos = $this->lancamentoFactory->getTotalImpostosItensSemAliquota($filial, $numLancamento);
                $nota = Notafiscal::findByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $this->environment->getNrOrgTrab());
                
                $lancamentoEntradaService = Dfe11000NfEntradaService::instanceService(
                    $this->entityManager, $this->novoCodigo, $this->environment,
                    $cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"]
                );
                
                $lancamentoEntradaService->salvaImpostosNotaFiscal($impostos);
            }

            $this->callProcedureFechamento($cabecalho, $dadosProcessFilial['DTPROCESSA'], $numLancEstoque);

            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($filial, $numLancEstoque, $organizacao);

            // NÃO COMENTAR ESTA LINHA EM HIPOTESE ALGUMA
            // CASO NECESSITE DE ALTERAÇÕES PROCURAR PEDRO MOUTINHO OU RAFAEL DIAS
            if (is_object($lanctoestoq))
                 $this->validacoesEstoqueService->verificaIrregularidades($lanctoestoq);
            
            $this->validaErrosEAvisos($filial, $numLancamento, $filialSelecionada, $forcaFechamento);
            
            $this->entityManager->refresh($notaFiscal);
            $vencimentos = $this->comparaVencimentoInfoGerado($filial, $numLancamento, $notaFiscal->getVrnotafisc(), $vencimentos, $organizacao, $cabecalho['IDENTRSAIDOP']);

            $notaFiscal->setIdstatus('P');
            
            if($cabecalho['IDENTRSAIDOP'] === 'S'){
                if(isset($cabecalho['CDFILIEMIT']) && isset($cabecalho['CDFILIRECE'])) {
                    if($cabecalho['CDFILIEMIT'] && $cabecalho['CDFILIRECE']) {
                        $errosReplicacao = $this->replicacaoAutomatica($notaFiscal, $dadosProcessFilial['DTPROCESSA']);
                        if($errosReplicacao){
                            $connection->rollback();
                            return array('ERROS_REPLICACAO' => $errosReplicacao);
                        }
                    }
                }
                if(!empty($vencimentos)) {
                    foreach($vencimentos as $vencimento) {
                        if(!$vencimento["DTATUAVENREC"] && ($vencimento["QTDIASVENREC"]  || $vencimento["QTDIASVENREC"] === 0)) {
                            $this->corrigirDataVencimento($cabecalho, $vencimento);
                        } 
                    }
                }
            }
            
            $notaFiscal->persist();
            $this->entityManager->flush();
            $this->entityManager->refresh($notaFiscal);
            
            $status = false;
            
            $connection->commit();
            
            try {
                $aprovAutops = $cabecalho["IDENTRSAIDOP"] == 'E' ? $this->lancamentoFactory->getAprovAutops($cabecalho['CDTIPOOPER']) : false;
                if( isset($cabecalho["IDNFGERADAVIAXML"]) ) {
                    $aprovacaoAutomatica = $this->lancamentoFactory->aprovaNfXmlAutomatica();
                    if($aprovacaoAutomatica) {
                        if( $aprovAutops || ($cabecalho["IDNFGERADAVIAXML"] === 'S' && $cabecalho["IDENTRSAIDOP"] == 'E') ) {
                            $response = new Response();
                            $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($this->environment->getNrOrg(), $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $cabecalho['IDENTRSAIDOP'], $response);
                            $status = empty($errosAprovacao);
                        }
                        else if($notaXml = $lancamentoEntradaService->verificaNotaXml($cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"])){
                            $response = new Response();
                            $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($this->environment->getNrOrg(), $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $cabecalho['IDENTRSAIDOP'], $response);
                            $status = empty($errosAprovacao);
                            
                        }
                    }else if( $aprovAutops ){
                            $response = new Response();
                            $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($this->environment->getNrOrg(), $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $cabecalho['IDENTRSAIDOP'], $response);
                            $status = empty($errosAprovacao);
                    }
                    else {
                        $status = false;
                    }
                    
                } else if( $aprovAutops ){
                    $response = new Response();
                    $errosAprovacao = $this->aprovacaoLancamentosService->aprovar($this->environment->getNrOrg(), $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $cabecalho['IDENTRSAIDOP'], $response);
                    $status = empty($errosAprovacao);
                }
            }
            catch(\Exception $e) {
                if(empty($errosAprovacao)) {
                    $errosAprovacao[0] = $e->getMessage();
                }
                $status = false;
            }
            
            // Realizar rotina de forma assincrona
            /*  if($cabecalho['IDENTRSAIDOP'] === 'E' && $cabecalho['NRACESSONFE']) {
                $this->manifestaNoLancamento($cabecalho['NRACESSONFE'], $cabecalho['CDEMPRESA'], $cabecalho["CDINSCESTA"]);
            }*/
            
            $infoNotaFiscal = array(
                'CABECALHO' => array(
                    'NRLANCTONF' => $notaFiscal->getNrlanctonf(),
                    'VRNOTAFISC' => $notaFiscal->getVrnotafisc()
                ),
                'IDREARATMANNF' => $cabecalho["IDENTRSAIDOP"] === 'E' ? $this->lancamentoFactory->getIdrearatmannf($cabecalho['CDTIPOOPER']) : 'N',
                'VENCIMENTO' => $vencimentos,
                'ERROS_APROVACAO' => isset($errosAprovacao) ? $errosAprovacao : array(),
                'STATUS' => ($status) ? 'A' : 'P'
            );
            
            return $infoNotaFiscal;
        } catch (\Exception $e) {
            if(strpos($e->getMessage(), 'ORA-') != -1) {
                $posicaoErro = strpos($e->getMessage(), 'ORA-');
                $erro = substr($e->getMessage(), $posicaoErro, strlen($e->getMessage()));
                $connection->rollback();
                throw new \Exception($erro);
            }
            $connection->rollback();
            throw $e;
        }
    }

    protected function corrigirDataVencimento($cabecalho, $vencimento) {
        $qtDiasVenRec = $vencimento["QTDIASVENREC"] . ' days';

        $dataSaida = \DateTime::createFromFormat('d/m/Y', substr($cabecalho["DTENTRSAID"], 0, 10));
        
        date_add($dataSaida, date_interval_create_from_date_string($qtDiasVenRec));

        $this->lancamentoFactory->updateDtVencimento($vencimento["CDFILIAL"], $vencimento["NRLANCTONF"], $vencimento["NRSEQVENREC"], $dataSaida->format('d/m/Y'));
        
    }
    
    public function validaFecharNota($cabecalho) {
        $produtosInativos = array();
        $isNotaAjuste =  $cabecalho["IDENTRSAIDOP"] == "S" && $cabecalho["IDNFAJUSTE"] == "S";
        $tipoOperacAtuEstoque = $this->lancamentoFactory->tipoOperacAtuEstoque($cabecalho["CDTIPOOPER"], $cabecalho["IDENTRSAIDOP"]);
    
        if($cabecalho["CDTIPOOPER"] != '03' && !$isNotaAjuste) {
            if(!$this->lancamentoFactory->validaExisteItem($cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"])) {
                throw new \Exception("Item(s) da nota fiscal devem ser informado(s)");
            };
        }
        
        $produtosSemLocal = $this->lancamentoFactory->validaLocalProduto($cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"]);
        
        if($cabecalho["NRACESSONFE"]) {
            $produtosInativos = $this->lancamentoFactory->getProdutosInativos($cabecalho["CDFILIAL"], $cabecalho["NRLANCTONF"]);
        }

        if(!empty($produtosInativos)) {
            $erros = array();
            $erro  = array("IDERROAVISO" => "B");
            foreach ($produtosInativos as $produto) {
                $codigo = $produto["CDARVPROD"];
                $nome   = $produto["NMPRODUTO"];
                if($produto["PRODESTOQATIVO"] == 'S') {
                    $erro["DSERROAVISO"] = "Produto: $codigo - $nome inativo";
                    if($produto["IDPRODATIV"] != 'A') {
                        $erro["DSERROAVISO"].=' para a filial';
                    } 
                }
                else {
                     $erro["DSERROAVISO"] = "Produto de estoque referente ao produto: $codigo - $nome inativo";
                }
                $erro["DSERROAVISO"].=".";
                $erro["NRSEQITPROD"] = $produto["NRSEQITPROD"];
                $this->errosAvisos[] = $erro;
            }
            throw new \Exception("ERRO");
        }

        if(!empty($produtosSemLocal) && $tipoOperacAtuEstoque && !$isNotaAjuste && $cabecalho["CDTIPOOPER"] != '03') {
            $erros = array();
            $erro  = array("IDERROAVISO" => "B");
            foreach ($produtosSemLocal as $produto) {
                $codigo = $produto["CDARVPROD"];
                $nome   = $produto["NMPRODUTO"];
                $erro["DSERROAVISO"] = "Almoxarifado e/ou Localização e/ou Lote não informado para o produto: $codigo - $nome";
                $erro["NRSEQITPROD"] = $produto["NRSEQITPROD"];
                $this->errosAvisos[] = $erro;
            }
            throw new \Exception("ERRO");
        }
    }
    
    public function manifestaNoLancamento($nrAcessoNfe, $cdEmpresa, $inscricao) {
        try {
            $paramManifesta = $this->lancamentoFactory->getParamManifesta();
        
            if(isset($paramManifesta[0])) {
            
                if($paramManifesta[0]["IDCIENCIAEMISS"]  == '2') $idManif = '4';
                if($paramManifesta[0]["IDCONFIRMAEMISS"] == '2') $idManif = '1';
    
                $manifesta = false;
                $notaDestinada = $this->lancamentoFactory->getNotaDestinada($nrAcessoNfe);
                
                if(isset($notaDestinada[0])) {
                    if(($notaDestinada[0]["IDSTATUSMANIF"] == 0 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 4 
                    || $notaDestinada[0]["IDSTATUSMANIF"] == 5 ) && $notaDestinada[0]["IDSTATUSMANIF"] != $idManif) {
                        $manifesta = true;
                    }
                }
                else {
                    $manifesta = true;
                }

                if($manifesta === true) {
                    // Manifesta nota
                    $nrOrg   = $this->environment->getNrOrg();
                    $config  = $this->manifestacaoService->montaConfigManifestacao($cdEmpresa, $nrOrg, $inscricao, '', true);
                    $nota    = array("NRACESSONFE" => $nrAcessoNfe, "NRNOTAFISC" => "", "NRLANCTONF" => "");
                    $retorno = $this->manifestacaoService->manifestarNfe(array($nota), $config, ($idManif == 4) ? "210210" : "210200", '');
                    if(isset($retorno["ERRO"])) throw new \Exception($retorno["ERRO"][0]["MOTIVO"]);
                }
                
            }
        }
        catch(\Exception $e) {}
        
    }

    public function replicacaoAutomatica($notafiscal, $dataProcessamento){
        $replicacao = new ReplicacaoDocumentosService($this->validacoesEstoqueService, $this->entityManager, $this->novoCodigo, $this->environment, $notafiscal, $dataProcessamento);
        return $replicacao->replicar();
    }

    public function replicacaoManual($notafiscal, $documentos, $itens) {
        $filialSelecionada = $notafiscal['CDFILIEMIT'];
        $organizacao = $this->environment->getNrOrgTrab();
        $dataProcessamento = $this->validacoesEstoqueService->retornaDadosProcessFilial($filialSelecionada, $organizacao)['DTPROCESSA'];
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($notafiscal['CDFILIAL'], $notafiscal['NRLANCTONF'], $organizacao);
        $replicacao = new ReplicacaoDocumentosService($this->validacoesEstoqueService, $this->entityManager, $this->novoCodigo, $this->environment, $notafiscal, $dataProcessamento, true);
        $errosReplicacao = $replicacao->replicar($documentos, $itens);
        if($errosReplicacao){
            return array('ERROS_REPLICACAO' => $errosReplicacao);
        }
        else {
            return array('SUCESSO' => array("SUCESSO"=>"SUCESSO"));
        }
    }

    public function procedureGeraNotaFiscalProduto(
        $filial, $numLancamento, $produto, $codEmissao, $descEmissao, $quantidade, $vrUnitario, $vrDesconto, 
        $almoxarifado = null, $localEstoque = null, $nrloteestq = null, $nrsublote = null, $cdservfatu = null,
        $idOrigemProd = null, $idAtuEstProd = 'S',  $vrICMSItOrig = null, $cdMarca = null, $cdservfili = null,
        $dsInfadicprod = null){
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_PRODUTO);
        
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $produto, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODEMIS', Param::PARAM_INPUT, $codEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSPRODEMIS', Param::PARAM_INPUT, $descEmissao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTITEMPROD', Param::PARAM_INPUT, $quantidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRUNITITEM', Param::PARAM_INPUT, $vrUnitario, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRDESCITEM', Param::PARAM_INPUT, $vrDesconto, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDFILIALITG', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINTEGRACAO', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRITEMINTEGR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCSTINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCFOPINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSUFXINFUSU', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $almoxarifado, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $localEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ', Param::PARAM_INPUT, $nrloteestq, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSUBLOTE', Param::PARAM_INPUT, $nrsublote, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDCONFORITEM', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDTPNFCREDEB', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QTVOLUITEM', Param::PARAM_INPUT, 0, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $cdservfili, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $cdMarca, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFATU', Param::PARAM_INPUT, $cdservfatu, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD', Param::PARAM_INPUT, $idOrigemProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTPROD', Param::PARAM_INPUT, $idAtuEstProd, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRICMSSTITORIG', Param::PARAM_INPUT, $vrICMSItOrig, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_DSJUSTIFICATIVA', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSINFADICPROD', Param::PARAM_INPUT, $dsInfadicprod, Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    
    public function procedureGetLote($P_IDUTILLOTE,$P_CDFILIAL,$P_CDALMOXARIFE,$P_CDLOCALESTOQ,$P_CDPRODUTO){

        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CONSULTA_LOTE);
        
        $procedure->addParam(new Param('P_IDUTILLOTE',   Param::PARAM_INPUT, $P_IDUTILLOTE,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $P_CDFILIAL,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDALMOXARIFE', Param::PARAM_INPUT, $P_CDALMOXARIFE, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDLOCALESTOQ', Param::PARAM_INPUT, $P_CDLOCALESTOQ, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO',    Param::PARAM_INPUT, $P_CDPRODUTO,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOTEESTQ',   Param::PARAM_OUTPUT,'',   Param::PARAM_TYPE_STR, 30));
        $procedure->addParam(new Param('P_NRSUBLOTE',    Param::PARAM_OUTPUT,'',    Param::PARAM_TYPE_STR, 30));
        
        $output = $procedure->execute();
        
        return $output;
    }
    
    public function ConsultaPosEstDia($P_CDFILIEMIT,  $P_CDALMOXARIFE, $P_CDLOCALESTOQ, $P_NRLOTEESTQ, $P_NRSUBLOTE, $P_DTPOSICAO, $P_INTPCUSTO,  $P_CDPRODUTO,$P_QTDE ){
        //Search CDPRODESTO;     
        
        $CDPRODESTO         = $this ->lancamentoFactory->consultaCDPRODUTO($P_CDPRODUTO);
        $CDPRODESTO_value   = $CDPRODESTO[0]["CDPRODESTO"];
        //var_dump($CDPRODESTO[0]["CDPRODESTO"]);
        
        
        //Search maxData POSESTDIA;
        $P_DTPOSIESTQ       = $this ->lancamentoFactory->consultaMaxData(
                                                                        $P_CDPRODUTO    ,
                                                                        $P_CDFILIEMIT   ,
                                                                        $P_CDALMOXARIFE ,
                                                                        $P_CDLOCALESTOQ ,
                                                                        $P_NRLOTEESTQ   ,
                                                                        $P_NRSUBLOTE    ,
                                                                        $CDPRODESTO_value,
                                                                        $P_DTPOSICAO    
                                                                    );
        $P_DTPOSIESTQ_value = $P_DTPOSIESTQ[0]["DTPOSIESTQ"];
        
        //if !isset($P_DTPOSIESTQ_value, product has not been released;
        if(!isset($P_DTPOSIESTQ_value)){
            //throw new \Exception('Produto não lançado nesta filial.');
            return $VrUnitario_value = null;
        }
        
        //$VrUnitario;
        $VrUnitario   = $this ->lancamentoFactory->VrUnitario (
                                                                    $P_CDPRODUTO, 
                                                                    $P_CDFILIEMIT, 
                                                                    $P_CDALMOXARIFE, 
                                                                    $P_CDLOCALESTOQ, 
                                                                    $P_NRLOTEESTQ, 
                                                                    $P_NRSUBLOTE, 
                                                                    $P_DTPOSICAO, 
                                                                    $P_INTPCUSTO, 
                                                                    $P_DTPOSIESTQ_value, 
                                                                    $P_QTDE
                                                                );
           
        
        if(!isset($VrUnitario[0]['VrUnitario'])){
            return $VrUnitario_value = null;
        }else{
            $VrUnitario_value = $VrUnitario[0]['VrUnitario'];
        }
        
        return $VrUnitario_value;
    }
    
    

    public function procedureGeraFechamentoSaida($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = NULL, $evolEstoque = 'S', $bloqTrans = NULL, $atuEstoq = 'N'){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_SAIDA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EVOLESTOQ', Param::PARAM_INPUT, $evolEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDBLOQTRANS', Param::PARAM_INPUT, $bloqTrans, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDATUESTOQ', Param::PARAM_INPUT, $atuEstoq, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    private function procedureGeraFechamentoEntrada($filial, $numLancamento, $dtProcessamento, $numLancEstoque, $operador = ''){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $organizacao = $this->environment->getNrOrgTrab();
        $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $numLancamento, $organizacao);
        $this->entityManager->refresh($notaFiscal);
        $notaFiscal->setIdstatus('P');
        $notaFiscal->persist();
        $this->entityManager->flush();
        $connection->commit();
            
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }

    /* Compara o vencimento infromado pelo usuario com o vencimento gerado pela procedure.
     * Se houver diferenca do valor informado pelo usuario com o valor calculado devera ser exibido a mensagem
     * para avisar o usuário desta diferenca. A unica excecao e quando o usuario informa somente uma parcela,
     * devendo assim realizar update.
     */
    private function comparaVencimentoInfoGerado( $filialLancamento, $numLancamento, $valorNota, array $vencimentosInfo, $organizacao, $entrada_saida){
        try {
            if ($entrada_saida === 'S' && count($vencimentosInfo) > 1) {
                $valorTotalInfo = $this->calcularValorTotalVencimentos($vencimentosInfo, $entrada_saida);
                $difeValorVenc = $valorNota - $valorTotalInfo;

                if ($difeValorVenc != 0)
                    throw new \Exception('A soma dos vencimentos difere do valor total da NF. Diferença ' . $difeValorVenc);
            }

            return $this->buscaVencimentos($filialLancamento, $numLancamento, $entrada_saida);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    private function criaArrayVencimento($conta, $entrada_saida){
        if (!is_array($conta))
            $conta = array($conta);

        $contas = array();

        if($entrada_saida == 'S'){
            foreach ($conta as $vencimento) {
                /** @var Contareceber $vencimento */
                $dtVencimento = is_object($vencimento->getDtatuavenrec()) ? $vencimento->getDtatuavenrec()->format('d/m/Y') : null;
                $contas[] = array(
                    'CDFILIAL' => $vencimento->getCdfilial(),
                    'NRLANCTONF' => $vencimento->getNrlanctonf(),
                    'NRPARCELAREC' => $vencimento->getNrparcelarec(),
                    'SEQVENCIMENTO' => $vencimento->getNrseqvenrec(), //Utiliazado para o salvamento local
                    'NRSEQVENREC' => $vencimento->getNrseqvenrec(),
                    'QTDIASVENREC' => $vencimento->getQtdiasvenrec(),
                    'DTATUAVENREC' => $dtVencimento,
                    'VRATUAREC' => $vencimento->getVratuarec(),
                    'IDSTATREC' => $vencimento->getIdstatrec()
                );
            }
        }
        else{
            foreach ($conta as $vencimento) {
                $codTipoContaPag = $vencimento->getCdtpctpagar();
                $organizacao = $this->environment->getNrOrgTrab();

                /** @var Contapagar $vencimento */
                $dtVencimento = is_object($vencimento->getDtatuavenpag()) ? $vencimento->getDtatuavenpag()->format('d/m/Y') : null;
                $dsTipoContaPag = null;
                $tipoContaPag = Tipoctpagar::findOneByCdtpctpagarAndNrorg($codTipoContaPag, $organizacao);

                if (is_object($tipoContaPag))
                    $dsTipoContaPag = $tipoContaPag->getDstpctpagar();

                $contas[] = array(
                    'CDFILIAL' => $vencimento->getCdfilial(),
                    'NRLANCTONF' => $vencimento->getNrlanctonf(),
                    'NRPARCELAPAG' => $vencimento->getNrparcelapag(),
                    'SEQVENCIMENTO' => $vencimento->getNrseqvenpag(), //Utiliazado para o salvamento local
                    'NRSEQVENPAG' => $vencimento->getNrseqvenpag(),
                    'QTDIASVENPAG' => $vencimento->getQtdiasvenpag(),
                    'DTATUAVENPAG' => $dtVencimento,
                    'VRATUAPAG' => $vencimento->getVratuapag(),
                    'IDSTATPAG' => $vencimento->getIdstatpag(),
                    'DSCONTAPAG' => $vencimento->getDscontapag(),
                    'CDTPCTPAGAR' => $codTipoContaPag,
                    'DSTPCTPAGAR' => $dsTipoContaPag,
                    'EXISTEBOLETO' => $this->lancamentoFactory->existeBoleto($vencimento->getCdfilial(), $vencimento->getNrlanbolpag())

                );
            }
        }

        $this->orderArray($contas, ($entrada_saida == 'S') ? 'DTATUAVENREC' : 'DTATUAVENPAG');

        return $contas;
    }

    
    
    
    protected function orderArray(&$array, $keyToOrder) {
        $sortArray = array(); 
        foreach($array as $e){ 
            foreach($e as $key=>$value){ 
                if(!isset($sortArray[$key])){ 
                    $sortArray[$key] = array(); 
                } 
                $sortArray[$key][] = $value; 
            } 
        } 

        if(!empty($array)) {
            array_multisort($sortArray[$keyToOrder], SORT_ASC, $array); 
        }
    }

    public function updateStatusNotafiscal($filial, $lancamento, $organizacao, $status){
        /** @var Notafiscal $notafiscal */
        $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);

        $notafiscal->setIdstatus($status);

        $this->entityManager->persist($notafiscal);
        $this->entityManager->flush();
    }

    public function updateVencimento($entrada_saida, $filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null, $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null, $tpContaReceber = null, $numParcelas = null, $descricao = null, $classFina = null, $operador = null, $ativo = null){
        
        try {
            if($entrada_saida == 'S'){
                return $this->updateContareceber($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem,
                    $vrAtual, $dtVencimento, $dtOrigemVencimento, $status, $numDias,
                    $tpContaReceber, $numParcelas, $descricao, $classFina,
                    $operador, $ativo);
            }
            else{
                return $this->updateContapagar($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem,
                    $vrAtual, $dtVencimento, $dtOrigemVencimento, $status, $numDias,
                    $tpContaReceber, $numParcelas, $descricao, $classFina,
                    $operador, $ativo);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function updateContareceber($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null,
                                        $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null,
                                        $tpContaReceber = null, $numParcelas = null, $descricao = null, $classFina = null,
                                        $operador = null, $ativo = null){
        /** @var Contareceber $contaReceber */
        $contaReceber = Contareceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrorg(
            $filial, $numLancamento, $sequencial, $organizacao
        );

        $values = [
            'numParcelas'        => [self::VALUE => $numParcelas,        self::VALUE_COLUMN => $contaReceber->getNrparcelarec()],
            'tpContaReceber'     => [self::VALUE => $tpContaReceber,     self::VALUE_COLUMN => $contaReceber->getCdtpctreceb()],
            'numDias'            => [self::VALUE => $numDias,            self::VALUE_COLUMN => $contaReceber->getQtdiasvenrec()],
            'vrOrigem'           => [self::VALUE => $vrOrigem,           self::VALUE_COLUMN => $contaReceber->getVrorigrec()],
            'vrAtual'            => [self::VALUE => $vrAtual,            self::VALUE_COLUMN => $contaReceber->getVratuarec()],
            'dtVencimento'       => [self::VALUE => $dtVencimento,       self::VALUE_COLUMN => $contaReceber->getDtatuavenrec()],
            'dtOrigemVencimento' => [self::VALUE => $dtOrigemVencimento, self::VALUE_COLUMN => $contaReceber->getDtorigvenrec()],
            'classFina'          => [self::VALUE => $classFina,          self::VALUE_COLUMN => $contaReceber->getCdclassfina()],
            'descricao'          => [self::VALUE => $descricao,          self::VALUE_COLUMN => $contaReceber->getDscontarec()],
            'status'             => [self::VALUE => $status,             self::VALUE_COLUMN => $contaReceber->getIdstatrec()],
            'operador'           => [self::VALUE => $operador,           self::VALUE_COLUMN => $contaReceber->getCdoperultatu()],
            'ativo'              => [self::VALUE => $ativo,              self::VALUE_COLUMN => $contaReceber->getIdativo()]
        ];

        foreach ($values as $key => $value) {
            $values[$key] = (!$value[self::VALUE]) ? $value[self::VALUE_COLUMN] : $value[self::VALUE];
        }

        /* extract() -> Verifica cada chave do array e vê se ele tem um válido nome de variável.
         *           -> Verifica por colisões com já existente variáveis.
         */
        extract($values);

        $contaReceber->setNrparcelarec($numParcelas);
        $contaReceber->setCdtpctreceb($tpContaReceber);
        $contaReceber->setQtdiasvenrec($numDias);
        $contaReceber->setVrorigrec($vrOrigem);
        $contaReceber->setVratuarec($vrAtual);
        $contaReceber->setDtatuavenrec(self::formataData($dtVencimento));
        $contaReceber->setDtorigvenrec(self::formataData($dtOrigemVencimento));
        $contaReceber->setCdclassfina($classFina);
        $contaReceber->setDscontarec($descricao);
        $contaReceber->setIdstatrec($status);
        $contaReceber->setNrorgultatu($organizacao);
        $contaReceber->setCdoperultatu($operador);
        $contaReceber->setDtultatu(DateUtil::getDataAtual());
        $contaReceber->setIdativo($ativo);

        $this->entityManager->persist($contaReceber);
        $this->entityManager->flush();

        return $contaReceber;
    }

    private function updateContapagar($filial, $numLancamento, $sequencial, $organizacao, $vrOrigem = null,
                                        $vrAtual = null, $dtVencimento = null, $dtOrigemVencimento = null, $status = null, $numDias = null,
                                        $tpContaPagar = null, $numParcelas = null, $descricao = null, $classFina = null,
                                        $operador = null, $ativo = null){
        /** @var Contapagar $contaPagar */
        $contaPagar = Contapagar::findOneByCdfilialAndNrlanctonfAndNrseqvenpagAndNrorg(
            $filial, $numLancamento, $sequencial, $organizacao
        );

        $values = [
            'numParcelas'        => [self::VALUE => $numParcelas,        self::VALUE_COLUMN => $contaPagar->getNrparcelapag()],
            'tpContaPagar'     => [self::VALUE => $tpContaPagar,     self::VALUE_COLUMN => $contaPagar->getCdtpctpagar()],
            'numDias'            => [self::VALUE => $numDias,            self::VALUE_COLUMN => $contaPagar->getQtdiasvenpag()],
            'vrOrigem'           => [self::VALUE => $vrOrigem,           self::VALUE_COLUMN => $contaPagar->getVrorigpag()],
            'vrAtual'            => [self::VALUE => $vrAtual,            self::VALUE_COLUMN => $contaPagar->getVratuapag()],
            'dtVencimento'       => [self::VALUE => $dtVencimento,       self::VALUE_COLUMN => $contaPagar->getDtatuavenpag()],
            'dtOrigemVencimento' => [self::VALUE => $dtOrigemVencimento, self::VALUE_COLUMN => $contaPagar->getDtorigvenpag()],
            'classFina'          => [self::VALUE => $classFina,          self::VALUE_COLUMN => $contaPagar->getCdclassfina()],
            'descricao'          => [self::VALUE => $descricao,          self::VALUE_COLUMN => $contaPagar->getDscontapag()],
            'status'             => [self::VALUE => $status,             self::VALUE_COLUMN => $contaPagar->getIdstatpag()],
            'operador'           => [self::VALUE => $operador,           self::VALUE_COLUMN => $contaPagar->getCdoperultatu()],
            'ativo'              => [self::VALUE => $ativo,              self::VALUE_COLUMN => $contaPagar->getIdativo()]
        ];

        foreach ($values as $key => $value) {
            $values[$key] = (!$value[self::VALUE]) ? $value[self::VALUE_COLUMN] : $value[self::VALUE];
        }

        /* extract() -> Verifica cada chave do array e vê se ele tem um válido nome de variável.
         *           -> Verifica por colisões com já existente variáveis.
         */
        extract($values);

        $contaPagar->setNrparcelapag($numParcelas);
        $contaPagar->setCdtpctpagar($tpContaPagar);
        $contaPagar->setQtdiasvenpag($numDias);
        $contaPagar->setVrorigpag($vrOrigem);
        $contaPagar->setVratuapag($vrAtual);
        $contaPagar->setDtatuavenpag(self::formataData($dtVencimento));
        $contaPagar->setDtorigvenpag(self::formataData($dtOrigemVencimento));
        $contaPagar->setCdclassfina($classFina);
        $contaPagar->setDscontapag($descricao);
        $contaPagar->setIdstatpag($status);
        $contaPagar->setNrorgultatu($organizacao);
        $contaPagar->setCdoperultatu($operador);
        $contaPagar->setDtultatu(DateUtil::getDataAtual());
        $contaPagar->setIdativo($ativo);

        $this->entityManager->persist($contaPagar);
        $this->entityManager->flush();

        return $contaPagar;
    }

    /**
     * @param array $vencimentosInfo
     * @param       $entrada_saida
     * @return int
     */
    private function calcularValorTotalVencimentos(array $vencimentosInfo, $entrada_saida){
        $valorTotalInfo = 0;
        
        if ($entrada_saida == 'S') {
            foreach ($vencimentosInfo as $vencInfo) {
                $valorTotalInfo += $vencInfo['VRATUAREC'];
            }
            return $valorTotalInfo;
        } else {
            foreach ($vencimentosInfo as $vencInfo) {
                $valorTotalInfo += $vencInfo['VRATUAPAG'];
            }
            return $valorTotalInfo;
        }
    }

    /**
     * @param $cabecalho
     * @param $dtProcessamento
     * @param $numLancEstoque
     * @return void
     */
    private function callProcedureFechamento($cabecalho, $dtProcessamento, $numLancEstoque){
        if(is_object($dtProcessamento))
            $dtProcessamento = $dtProcessamento->format('d/m/Y');

        //@TODO verificar se precisa passar o parametro operador para a procedure

        if($cabecalho['IDENTRSAIDOP'] == 'S'){
            $this->procedureGeraFechamentoSaida(
                $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $dtProcessamento, $numLancEstoque
            );
        }

        else{
            $this->procedureGeraFechamentoEntrada(
                $cabecalho['CDFILIAL'], $cabecalho['NRLANCTONF'], $dtProcessamento, $numLancEstoque
            );
        }

        $this->entityManager->flush();
    }

    public function calculaImpostoItemProduto($cabecalho, $item, $impostos = false) {
            $verificaImpostos = is_array($impostos) || $impostos;
            $filial = $cabecalho['CDFILIAL'];
            $lancamento = $cabecalho['NRLANCTONF'];
            $empresa = $cabecalho['CDEMPRESA'];
            $inscesta = $cabecalho['CDINSCESTA'];
            $tipoOperacao = $cabecalho['CDTIPOOPER'];
            $fornecedor = $cabecalho['CDFORNECED'];
            $dtEntrada = $cabecalho['DTENTRSAID'];
            $entrada_saida = $cabecalho['IDENTRSAIDOP'];
            $idorigdestnf = $cabecalho['IDORIGDESTNF'];
            $filialRece = $cabecalho['CDFILIRECE'];
            // $cdfiliemit = $cabecalho['CDFILIEMIT'];
            $cdfiliemit = null;
            $cliente = $cabecalho['CDCLIENTE'];

            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();
            $dtInclusao = DateUtil::getDataAtual();

            $seqItem = $item['NRSEQITPROD'];
            $cdarvprod = $item['CDARVPROD'];
            $valorProduto = $item['VRTOTAITEM'];
            $sgEstado = isset($cabecalho['SGESTADONOTA']) ? $cabecalho['SGESTADONOTA'] : null;
            $cdMunicipio = isset($cabecalho['CDMUNICIPIONOTA']) ? $cabecalho['CDMUNICIPIONOTA'] : null;

            $idgernfigualxml = $cabecalho['IDGERNFIGUALXML'];
            $isLancManual = $cabecalho['NRACESSONFE'] == null;

            $impostosTemporarios = array();
            $trabitproimpostos = array();
   
            $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrseqimpoitAndNrorg($filial, $lancamento, $seqItem, $organizacao);
            
            if (is_array($trabitproimpos)) {
                foreach ($trabitproimpos as $trabImposto) {
                    if(!$verificaImpostos || !in_array($trabImposto->getCdimpostoit(), $impostos)) {
                        $trabImposto->delete();
                    }
                }
            }
            
            $this->entityManager->flush();

            $grupofiscal = $this->callConsultaGrupofiscal(
                $idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao
            );

            $idinterest = $this->getIdinterest(
                $empresa, $inscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao
            );
            
            $cdgrpfiscsv = '';

            $this->callProcedureImpostoEntrada(
                $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
                $lancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
                $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
                $cliente, $valorProduto, $sgEstado, $cdMunicipio
            );
            //$aliqtempprod = Aliqtempprod::findByNraliqtempAndNrorg($filial . $lancamento, $organizacao);
            $aliqtempprod =  $this->lancamentoFactory->getAliqtempprod($filial . $lancamento, $organizacao);
            foreach ($aliqtempprod as $aliq) {
                if(!$verificaImpostos || !in_array($aliq["CDIMPOTEMP"], $impostos)) {
                    $impostosTemporarios[$aliq["CDIMPOTEMP"]] = array(
                        'CDIMPOTEMP' => $aliq["CDIMPOTEMP"],
                        'VRALIQIMPOIT' => $aliq["VRALIQUTMP"],
                        'VRADICALIQFECP' => $aliq["VRADICALIQFECP"]
                    );
                    
                }
                $this->lancamentoFactory->deleteAliqtempprod($filial . $lancamento, $aliq["CDIMPOTEMP"] ,$organizacao);
            }
             

            $incitemp = Incitemp::findByNrincitempAndNrorg($filial . $lancamento, $organizacao);
            foreach ($incitemp as $inci) {
                if(!$verificaImpostos || !in_array($inci->getCdimpotemp(), $impostos)) {
                    if(isset($impostosTemporarios[$inci->getCdimpotemp()]))
                        $impostosTemporarios[$inci->getCdimpotemp()]['VRIMPOSTOIT'] = 0;
                }
                $inci->delete();
                
            }

            foreach ($impostosTemporarios as $imposto) {
                $cdimposto = $imposto['CDIMPOTEMP'];
                $vrimpostoit = $imposto['VRIMPOSTOIT'];
                $vraliqitem = $imposto['VRALIQIMPOIT'];
                $vradicaliqfecp = $imposto["VRADICALIQFECP"];
                $alteracaoUser = 'N';
                
                $idimportaimp =  Imposto::findOneByCdimpostoAndNrorg($cdimposto, $organizacao);
                $idvalorinfo = (($idgernfigualxml == 'S' || $idimportaimp->getIdimportaimpxml() == 'S' ) && !$isLancManual) ? 'S' : 'N';              
                
                $this->novoTrabitproimpos(
                    $filial, $lancamento, $seqItem, $cdimposto, $organizacao, $operador, $dtInclusao,
                    $vrimpostoit, $vraliqitem, $alteracaoUser, $idvalorinfo, $vradicaliqfecp
                );
            }
            
            $this->entityManager->flush();

            foreach ($impostosTemporarios as &$impos) {
                $cdimposto = $impos['CDIMPOTEMP'];
                $imposto = Imposto::findOneByCdimpostoAndNrorg($cdimposto, $organizacao);
                $impos['NMIMPOSTO'] = $imposto->getNmimposto();
                $impos['SGIMPOSTO'] = $imposto->getSgimposto();
                
                $impos['CDFISCALIT'] =  ($impos['VRIMPOSTOIT'] > 0) ? 1 : 2;
                $impos['CDREGRAIT'] = 0;
                $impos['IDMODBCITNFE'] = 0;
                $impos['IDMODBCSTITNFE'] = 0;
                $impos['IDTRIMPSTTI'] = 0;
                $impos['NRITPROIMPOS'] = 1;
                $impos['VRCONTITIMP']  = 0;
                $impos['VRPERREDBCIT'] = 0;
                $impos['VRBASECALCIT'] = 0;
                $impos['VRPARDEDIT'] = 0;
                $impos['CDNATUBCALPCIT'] = 0;
                $impos['CDTPCREDPICOITE'] = 0;
            }
            
            return $impostosTemporarios;

    }
    
    /*public function calculaImpostoItens($cabecalho) {
        try {
            //TODO refatorarrrrrrrrrrrrr
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $filial = $cabecalho['CDFILIAL'];
            $lancamento = $cabecalho['NRLANCTONF'];
            $empresa = $cabecalho['CDEMPRESA'];
            $inscesta = $cabecalho['CDINSCESTA'];
            $tipoOperacao = $cabecalho['CDTIPOOPER'];
            $fornecedor = $cabecalho['CDFORNECED'];
            $dtEntrada = $cabecalho['DTENTRSAID'];
            $entrada_saida = $cabecalho['IDENTRSAIDOP'];
            $idorigdestnf = $cabecalho['IDORIGDESTNF'];
            $filialRece = $cabecalho['CDFILIRECE'];
            // $cdfiliemit = $cabecalho['CDFILIEMIT'];
            $cdfiliemit = null;
            $cliente = $cabecalho['CDCLIENTE'];

            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();
            $dtInclusao = DateUtil::getDataAtual();

            $impostosTemporarios = array();
            $trabitproimpostos = array();
            $itensImpostos = Itemproimpos::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
            if (is_array($itensImpostos)) {
                foreach ($itensImpostos as $itemImposto) {
                    $imposto = $itemImposto->getCdimposto();
                    $seqItem = $itemImposto->getNrseqitprod();

                    $vraliqitem = $itemImposto->getVraliqitem();
                    $vrimpostoit = $itemImposto->getVrimpostoit();
                    $alteracaoUser = $itemImposto->getIdavisimpit();

                    $trabitproimpostos[] = $this->novoTrabitproimpos(
                        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $dtInclusao,
                        $vrimpostoit, $vraliqitem, $alteracaoUser,$vradicaliqfecp
                    );
                    $itemImposto->delete();
                }
                $this->entityManager->flush();
            }
            
            $itens = $this->lancamentoFactory->getItensNota($filial, $lancamento);
        
            foreach($itens as $item) {
                
                $seqItem = $item['NRSEQITPROD'];
                $cdarvprod = $item['CDARVPROD'];
                $valorProduto = $item['VRTOTAITEM'];
                
                $trabitproimpos = Trabitproimpos::findByCdfilimpoitAndNrlancimpoitAndNrseqimpoitAndNrorg($filial, $lancamento, $seqItem, $organizacao);
                if (is_array($trabitproimpos)) {
                    foreach ($trabitproimpos as $trabImposto) {
                        $trabImposto->delete();
                    }
                }
                $this->entityManager->flush();

                $grupofiscal = $this->callConsultaGrupofiscal(
                    $idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao
                );

                $idinterest = $this->getIdinterest(
                    $empresa, $inscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao
                );
                
                $cdgrpfiscsv = '';

                $this->callProcedureImpostoEntrada(
                    $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
                    $lancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
                    $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
                    $cliente, $valorProduto
                );

                $aliqtempprod = Aliqtempprod::findByNraliqtempAndNrorg($filial . $lancamento, $organizacao);
                foreach ($aliqtempprod as $aliq) {
                    $impostosTemporarios[$aliq->getCdimpotemp()] = array(
                        'CDIMPOTEMP' => $aliq->getCdimpotemp(),
                        'VRALIQIMPOIT' => $aliq->getVraliqutmp()
                    );
                    $aliq->delete();
                }
                 

                $incitemp = Incitemp::findByNrincitempAndNrorg($filial . $lancamento, $organizacao);
                foreach ($incitemp as $inci) {
                    $impostosTemporarios[$inci->getCdimpotemp()]['VRIMPOSTOIT'] = 0;
                    $inci->delete();
                }

                foreach ($impostosTemporarios as $imposto) {
                    $cdimposto = $imposto['CDIMPOTEMP'];
                    $vrimpostoit = $imposto['VRIMPOSTOIT'];
                    $vraliqitem = $imposto['VRALIQIMPOIT'];
                    $alteracaoUser = 'N';

                    $this->novoTrabitproimpos(
                        $filial, $lancamento, $seqItem, $cdimposto, $organizacao, $operador, $dtInclusao,
                        $vrimpostoit, $vraliqitem, $alteracaoUser, $vradicaliqfecp
                    );
                }
                $this->entityManager->flush();
            } 
            

            
            $connection->commit();

            foreach ($impostosTemporarios as &$impos) {
                $cdimposto = $impos['CDIMPOTEMP'];
                $imposto = Imposto::findOneByCdimpostoAndNrorg($cdimposto, $organizacao);
                $impos['NMIMPOSTO'] = $imposto->getNmimposto();
                $impos['SGIMPOSTO'] = $imposto->getSgimposto();
            }

            return $impostosTemporarios;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }*/

    private function callProcedureImpostoEntrada(
        $empresa, $inscesta, $tipoOperacao, $fornecedor, $filial,
        $seqLancamento, $dtEntrada, $entrada_saida, $operador, $grupofiscal,
        $idinterest, $cdarvprod, $cdgrpfiscsv, $idorigdestnf, $filialRece, $cdfiliemit,
        $cliente, $valorProduto, $sgEstado, $cdMunicipio
    ) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'IMPOSTO_ENTRADA_PRODUTO');
        $procedure->addParam(new Param('P_EMPRESA', Param::PARAM_INPUT, $empresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA', Param::PARAM_INPUT, $inscesta, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPOOPER', Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $seqLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID', Param::PARAM_INPUT, $dtEntrada, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ENTRSAID', Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPOFISC', Param::PARAM_INPUT, $grupofiscal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDINTEREST', Param::PARAM_INPUT, $idinterest, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDARVPROD', Param::PARAM_INPUT, $cdarvprod, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRPFISCSV', Param::PARAM_INPUT, $cdgrpfiscsv, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDORIGDESTNF', Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE', Param::PARAM_INPUT, $filialRece, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT', Param::PARAM_INPUT, $cdfiliemit, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $cliente, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPRODUTO', Param::PARAM_INPUT, $valorProduto, Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDORIGEMPROD',    Param::PARAM_INPUT, 'N',  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGESTADO',        Param::PARAM_INPUT, $sgEstado,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMUNICIPIO',     Param::PARAM_INPUT, $cdMunicipio,   Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }

    public function callConsultaGrupofiscal($idorigdestnf, $fornecedor, $filial, $entrada_saida, $tipoOperacao) {
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CONSULTA_GRUPOFISCAL');
        $procedure->addParam(new Param("P_IDORIGDESTNF", Param::PARAM_INPUT, $idorigdestnf, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDORIGDESTNF", Param::PARAM_INPUT, $fornecedor, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIFATUR", Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDENTRSAIDOP", Param::PARAM_INPUT, $entrada_saida, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER", Param::PARAM_INPUT, $tipoOperacao, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC", Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_STR, 10));
        $output = $procedure->execute();

        return $output['P_CDGRUPOFISC'];
    }

    public function getIdinterest($empresa, $cdinscesta, $cliente, $fornecedor, $filial, $idorigdestnf, $organizacao) {
        $idinterest = 'N';

        $endeEmpresa = Empresa::getRepository()->getEstadoPais(
            $empresa, $cdinscesta, $organizacao
        );

        switch ($idorigdestnf) {
            case 'F':
                $endeForn = Endeforn::getRepository()->getEstadoPais($fornecedor, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeForn['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeForn['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'C':
                $endeClie = Endeclie::getRepository()->getEstadoPais($cliente, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeClie['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeClie['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
            case 'L':
                $endeFili = Endefili::getRepository()->getEstadoPais($filial, $organizacao);
                if ($endeEmpresa['CDPAIS'] == $endeFili['CDPAIS'] && $endeEmpresa['SGESTADO'] != $endeFili['SGESTADO']) {
                    $idinterest = 'S';
                }
                break;
        }
        return $idinterest;
    }

    public function novoTrabitproimpos(
        $filial, $lancamento, $seqItem, $imposto, $organizacao, $operador, $dtInclusao,
        $vrimpostoit, $vraliqitem, $alteracaoUser, $idvalorinfo = null, $vradicaliqfecp = 0, $nrseqitpro = 1, $vrcontimpoit = 0, $vrredbcimpoit = 0, $vrbcalcimpoit = 0, $vrpdedimpoit = 0,
        $idimpdesotrabit = 'N', $idmotdesotrabit = '9', $iddifaliqcuspti = 'N'
    ) {
        $itemImpostoTemp = new Trabitproimpos();

        $cdfiscimpoit = ($vrimpostoit > 0)? 1 : 2;

        $itemImpostoTemp->setCdfilimpoit($filial);
        $itemImpostoTemp->setNrlancimpoit($lancamento);
        $itemImpostoTemp->setNrseqimpoit($seqItem);
        $itemImpostoTemp->setCdimpostoit($imposto);
        $itemImpostoTemp->setNrseqitpro($nrseqitpro);
        $itemImpostoTemp->setNrorg($organizacao);

        $itemImpostoTemp->setCdfiscimpoit($cdfiscimpoit);

        $itemImpostoTemp->setCdoperinclusao($operador);
        $itemImpostoTemp->setDtinclusao($dtInclusao);

        $itemImpostoTemp->setVrcontimpoit($vrcontimpoit);
        $itemImpostoTemp->setVrredbcimpoit($vrredbcimpoit);
        $itemImpostoTemp->setVrbcalcimpoit($vrbcalcimpoit);
        $itemImpostoTemp->setVrpdedimpoit($vrpdedimpoit);

        $itemImpostoTemp->setIdimpdesotrabit($idimpdesotrabit);
        $itemImpostoTemp->setIdmotdesotrabit($idmotdesotrabit);
        $itemImpostoTemp->setIddifaliqcuspti($iddifaliqcuspti);

        $itemImpostoTemp->setVrimposit($vrimpostoit);
        $itemImpostoTemp->setVraliqimpoit($vraliqitem);
        $itemImpostoTemp->setIdavisimpoit($alteracaoUser);
        
        $itemImpostoTemp->setIdvalorinfo($idvalorinfo);
        $itemImpostoTemp->persist();
        $this->entityManager->flush();
        // existem campos que não exitem na classe Trabitproimpos e por questoes de urgencias serão updatados manualmente;
        $params = array(
            ':P_CDFILIMPOIT' => $filial,
            ':P_NRLANCIMPOIT' => $itemImpostoTemp->getNrlancimpoit(),
            ':P_NRSEQIMPOIT' => $seqItem,
            ':P_CDIMPOSTOIT'=> $imposto,
            ':P_NRSEQITPRO' =>$nrseqitpro,
            ':P_NRORG' =>$organizacao,
            ':P_CDOPERINCLUSAO' =>$operador,
            ':P_VRADICALIQFECP' =>$vradicaliqfecp
        );
        
        $this->lancamentoFactory->updateItemImposTemp($params); 
        return $itemImpostoTemp;
    }

    public function salvaItensAf(array $afSelecionadas) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $organizacao = $this->environment->getNrOrgTrab();
            $nrseqitprod = 0;
            $arrayProdutos = array();

            foreach ($afSelecionadas as $af) {
                $filial = $af['CDFILIAL'];
                $seqLancamento = $af['NRLANCTONF'];

                $filialDestAf = $af['CDFILDEST'];
                $nrAutoForn = $af['NRAUTOFORN'];
                $itensAf = \Geral\Entities\Itemautf::findByCdfildestAndNrautoforn($filialDestAf, $nrAutoForn);
                if (is_array($itensAf)) {
                    foreach ($itensAf as $item) {
                        $produto = Produto::findOneByCdprodutoAndNrorg($item->getCdproduto(), $organizacao);

                        $nrseqitprod = str_pad(++$nrseqitprod, 4, '0', STR_PAD_LEFT);
                        $cdproduto = $item->getCdproduto();
                        $dtprogentaf = $item->getDtprogentaf()->format('d/m/Y');
                        $qtitemprod = floatval($item->getQtprogentaf()) - floatval($item->getQtefetentraf());

                        $cdprodemis = $cdproduto;
                        $dsprodemis = $produto->getNmproduto();

                        $vrbrutitem = $item->getVrpreuniaf();
                        $vrunititem = $item->getVrpreuniaf();
                        $vrdescitem = floatval($item->getVrdescitmaf());
                        $vrtotaitem = $vrunititem * $qtitemprod;

                        $arrayProdutos[] = $this->salvaItemproduto(
                            $filial, $seqLancamento, $nrseqitprod, $cdproduto, $cdprodemis,
                            $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
                            $vrtotaitem
                        );
                    }
                }
            }

            //$connection->commit();
            $arrayProdutos = $this->criaArrayProdutos($arrayProdutos);

            // Inserir na NFRELACAF o relacionamento da af com a nota

            return $arrayProdutos;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function salvaItemproduto(
        $filial, $seqLancamento, $nrseqitprod, $cdproduto, $cdprodemis,
        $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
        $vrtotaitem, $cdalmoxarife = self::ESTOQUE_SEPARADOR, $cdlocalestoq = self::ESTOQUE_SEPARADOR,
        $nrloteestq = self::ESTOQUE_SEPARADOR, $nrsublote = self::ESTOQUE_SEPARADOR,
        $idatuvrestoq = 'S', $vrcontitem = 0, $idconforitem = 'S'
    ) {
        try {
            $organizacao = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();
            $dtInclusao = DateUtil::getDataAtual();

            $itemproduto = new Itemproduto();
            $itemproduto->setCdfilial($filial);
            $itemproduto->setNrlanctonf($seqLancamento);
            $itemproduto->setNrseqitprod($nrseqitprod);
            $itemproduto->setCdproduto($cdproduto);
            $itemproduto->setCdprodemis($cdprodemis);
            $itemproduto->setDsprodemis($dsprodemis);
            $itemproduto->setQtitemprod($qtitemprod);
            $itemproduto->setVrbrutitem($vrbrutitem);
            $itemproduto->setVrunititem($vrunititem);
            $itemproduto->setVrdescitem($vrdescitem);
            $itemproduto->setVrtotaitem($vrtotaitem);
            $itemproduto->setCdalmoxarife($cdalmoxarife);
            $itemproduto->setCdlocalestoq($cdlocalestoq);
            $itemproduto->setNrloteestq($nrloteestq);
            $itemproduto->setNrsublote($nrsublote);
            $itemproduto->setIdatuvrestoq($idatuvrestoq);
            $itemproduto->setVrcontitem($vrcontitem);
            $itemproduto->setIdconforitem($idconforitem);

            $itemproduto->setNrorg($organizacao);
            $itemproduto->setNrorginclusao($organizacao);
            $itemproduto->setNrorgultatu($organizacao);
            $itemproduto->setCdoperinclusao($operador);
            $itemproduto->setCdoperultatu($operador);
            $itemproduto->setDtinclusao(DateUtil::getDataAtual());
            $itemproduto->setDtultatu(DateUtil::getDataAtual());

            $itemproduto->persist();
            $this->entityManager->flush();
            
            return $itemproduto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaExclusaoNotaUtilizaAf($filial, $lancamento) {
        $lancamentoSaidaService = Dfs11000NfSaidaService::instanceService(
            $this->entityManager, $this->novoCodigo, $this->environment,
            $filial, $lancamento, $this->environment->getNrOrgTrab()
        );

        return $lancamentoSaidaService->atualizarStatusAFdevolucaoExclusao();
    }

    private function validaFechamentoNotaUtilizaAf(array $cabecalho) {
        $entradaSaida = $cabecalho['IDENTRSAIDOP'];

        $organizacao = $this->environment->getNrOrgTrab();
        $filial = $cabecalho['CDFILIAL'];
        $lancamento = $cabecalho['NRLANCTONF'];
        
        if ($entradaSaida === Notafiscal::NOTA_FISCAL_SAIDA){
 
            $lancamentoSaidaService = Dfs11000NfSaidaService::instanceService(
                $this->entityManager, $this->novoCodigo, $this->environment,
                $filial, $lancamento, $organizacao
            );
            return $lancamentoSaidaService->atualizarStatusAFdevolucaoAlteracao($cabecalho['IDOPERDEVONF']);

            
        }

/*        $lancamentoEntradaService = Dfe11000NfEntradaService::instanceService(
            $this->entityManager, $this->novoCodigo, $this->environment,
            $filial, $lancamento
        );

        $lancamentoEntradaService->atualizaStatusAF($filial, $lancamento, $organizacao, 'I');*/
    }

    private function validaErrosEAvisos($filial, $lancamento, $filialMovimentada, $forcaFechamento) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $confereErrosAvisosNota = new ConfereErrosAvisosNota($this->entityManager, $filial, $lancamento);
            $this->errosAvisos = $confereErrosAvisosNota->confereErros($filialMovimentada);

            if (count($this->errosAvisos) > 0) {

                $bloqueia = false;
  
                foreach ($this->errosAvisos as $erro) {
                   if($erro["IDERROAVISO"] == 'B') $bloqueia = true;
                }
                if($bloqueia) throw new \Exception(json_encode($this->errosAvisos, JSON_FORCE_OBJECT));
                else if($forcaFechamento == false) {
                    throw new \Exception(json_encode($this->errosAvisos, JSON_FORCE_OBJECT));
                }           
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function geraCorpoNotaEntrada($cdfilial, $cdfilirece, $cdempresa, $cdtipooper, $cdserieentr, $cdcliente, $sistema, $atualizaestoque, 
        $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq,
        $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $vrnotafisc, $nrlancorig, $dtlancmennf, $dtemissao, $nrnotafisc, $nracessonfe, $cdmododocfisc) {
        
        $dtemissao  = str_replace("-", "/", $dtemissao);
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO_ENTRADA);
        
        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIRECE',      Param::PARAM_INPUT, $cdfilirece,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIEENTR',     Param::PARAM_INPUT, $cdserieentr,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRNOTAFISC',      Param::PARAM_INPUT, $vrnotafisc,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRNOTAFISC',      Param::PARAM_INPUT, $nrnotafisc,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTEMISSAO',       Param::PARAM_INPUT, $dtemissao,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRACESSONFE',     Param::PARAM_INPUT, $nracessonfe,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMODDOCFISC',    Param::PARAM_INPUT, $cdmododocfisc,     Param::PARAM_TYPE_STR));
        
        return $procedure->execute();
        
    }
    
    public function geraCorpoNota($cdfilial, $cdfiliemit, $cdempresa, $cdtipooper, $cdserie, $cdcliente, $sistema, $atualizaestoque, $nrlanctonf, $pesstrans, $insctrans, $placaveiculo, $ufveiculo, $pgtofrete, $quantidadedevolu, $espefvolu, $marcvolu, $numevolu, $nrlancestoq, $dtentrsaid, $vrbrutovolu, $vrliquidovolu, $nrlancorig = null, $dtlancmennf = null) {
        $dtentrsaid = str_replace("-", "/", $dtentrsaid);
        
        $prxLanctoNf    = NULL;
        $prxLanctoEstoq = NULL;
        
        /*$procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_GERA_NOTA_CORPO);

        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT, $cdfilial,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIEMIT',      Param::PARAM_INPUT, $cdfiliemit,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDEMPRESA',       Param::PARAM_INPUT, $cdempresa,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOOPER',      Param::PARAM_INPUT, $cdtipooper,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERIE',         Param::PARAM_INPUT, $cdserie,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCLIENTE',       Param::PARAM_INPUT, $cdcliente,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRSAID',      Param::PARAM_INPUT, $dtentrsaid,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSISTEMA',       Param::PARAM_INPUT, $sistema,           Param::PARAM_TYPE_STR));

        $procedure->addParam(new Param('P_PRXLANCTONF',     Param::PARAM_OUTPUT,$prxLanctoNf,       Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_PRXLANCESTQ',     Param::PARAM_OUTPUT,$prxLanctoEstoq,    Param::PARAM_TYPE_STR, 10));
        
        $procedure->addParam(new Param('P_ATUAESTOQUE',     Param::PARAM_INPUT, $atualizaestoque,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRAPURACFAT',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF',      Param::PARAM_INPUT, $nrlanctonf,        Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_IDTPIJURTRAN',    Param::PARAM_INPUT, $pesstrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRINSJURTRAN',    Param::PARAM_INPUT, $insctrans,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRPLACAVEIC',     Param::PARAM_INPUT, $placaveiculo,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SGUFVEIC',        Param::PARAM_INPUT, $ufveiculo,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPGTOFRETE',     Param::PARAM_INPUT, $pgtofrete,         Param::PARAM_TYPE_STR));
        
        
        $procedure->addParam(new Param('P_DSQTDEVOLU',      Param::PARAM_INPUT, $quantidadedevolu,  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSESPECVOLU',     Param::PARAM_INPUT, $espefvolu,         Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSMARCAVOLU',     Param::PARAM_INPUT, $marcvolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSNUMEVOLU',      Param::PARAM_INPUT, $numevolu,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPBRUTOVOLU',    Param::PARAM_INPUT, $vrbrutovolu,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRPLIQUVOLU',     Param::PARAM_INPUT, $vrliquidovolu,     Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param('P_NRLANCESTQ',      Param::PARAM_INPUT, $nrlancestoq,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCORIG',      Param::PARAM_INPUT, $nrlancorig,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTLANCAMENNF',    Param::PARAM_INPUT, $dtlancmennf,       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDGEROUTDOCU',    Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTCOMPFATUR',     Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPTOVNDFATURNF', Param::PARAM_INPUT, NULL,               Param::PARAM_TYPE_STR));
        
        return $procedure->execute();*/
    }
    
    private function getDataComBarras($data) {
        $data = str_replace("/", "-", $data);
        return date('d-m-Y', strtotime($data));
    }
    
    public function atualizaStatusIntgNotaFiscal($idStatusNfe, $nrAcessoNfe){
        $sql = "UPDATE INTG_NOTAFISCAL
                SET IDSTATUSNFE = :P_IDSTATUSNFE
                WHERE NRACESSONFE = :P_NRACESSONFE";
               
        $params = array("P_IDSTATUSNFE" => $idStatusNfe,
                        "P_NRACESSONFE" => $nrAcessoNfe);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);        
    }
    
    public function atualizaIntgNotaFiscal($idStatusNfe, $idImportada, $idUpdataData, $cdFiliRece, $cdFilial, $nrLanctonf, $nrAcessoNfe) {
        $sql = "UPDATE INTG_NOTAFISCAL
                 SET IDSTATUSNFE = :P_IDSTATUSNFE,
                     IDIMPORTADA = :P_IDIMPORTADA,
                     CDFILIRECE  = DECODE(:P_IDUPDATDATA, 'S', :P_CDFILIRECE, CDFILIRECE),
              
                       DTUTILIZAXML = DECODE(:P_IDUPDATDATA, 'S', SYSDATE, NULL),
                     CDFILIAL     = :P_CDFILIAL,
                     NRLANCTONF   = :P_NRLANCTONF
               WHERE NRACESSONFE = :P_NRACESSONFE";
               
        $params = array("P_IDSTATUSNFE" => $idStatusNfe,
                        "P_IDIMPORTADA" => $idImportada,
                        "P_IDUPDATDATA" => $idUpdataData,
                        "P_CDFILIRECE"  => $cdFiliRece,
                        "P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf,
                        "P_NRACESSONFE" => $nrAcessoNfe);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);        
    }
    
    public function insereHistoricoXML($nrAcessoNfe, $nrSeqHistor, $dtHistorico, $idTipoOper, $idStatus, $nrProcoloNFe, $dsOperacao, $cdFilial, $nrLanctonf, $cdOperador, $nrOrg) {
    	$sql = "INSERT INTO HISTXMLNFE(NRACESSONFE, NRSEQHISTOR, DTHISTORICO, IDTIPOOPER,
                               IDSTATUS, NRPROTOCOLONFE, DSOPERACAO, CDFILIAL,
                             NRLANCTONF, CDOPERADOR, NRORG)
               VALUES (:NRACESSONFE, :NRSEQHISTOR, :DTHISTORICO, :IDTIPOOPER,
            	:IDSTATUS, :NRPROTOCOLONFE, :DSOPERACAO, :CDFILIAL,
            	:NRLANCTONF, :CDOPERADOR, :NRORG)";
              
              
        $params = array("NRACESSONFE"		=> $nrAcessoNfe,
						"NRSEQHISTOR"		=> $nrSeqHistor, 
						"DTHISTORICO"		=> $dtHistorico,
						"IDTIPOOPER"		=> $idTipoOper,
						"IDSTATUS"			=> $idStatus,
						"NRPROTOCOLONFE"	=> $nrProcoloNFe,
						"DSOPERACAO"		=> $dsOperacao,
						"CDFILIAL"			=> $cdFilial,
						"NRLANCTONF"		=> $nrLanctonf,
						"CDOPERADOR"		=> $cdOperador,
						"NRORG"				=> $nrOrg);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getSequencialHistoricoXML($nrAcessoNfe) {

    	$sql = "SELECT MAX(NVL(NRSEQHISTOR,0)) AS NRSEQHISTOR
		        FROM HISTXMLNFE
		       WHERE NRACESSONFE = :P_NRACESSONFE";
		$params = array("P_NRACESSONFE"  => $nrAcessoNfe);
	    
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function atualizaNotaDestinada($filial, $lancamento, $nracessonfe) {
        $sql = "UPDATE NOTADESTINADA
                SET CDFILIAL      = :P_CDFILIAL,
                  NRLANCTONF      = :P_NRLANCTONF
                WHERE NRACESSONFE = :P_NRACESSONFE";
                
            
		$params = array("P_CDFILIAL"		=> $filial,
						"P_NRLANCTONF"		=> $lancamento,
						"P_NRACESSONFE"     => $nracessonfe);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);

    }
    
    public function geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao){
        if($consContaRateio['IDCONSCTBRAT'] == 'S'){ //Considera tipo de conta para geração de rateio contabil
            $contapagar = $this->buscaVencimentos($filial, $lancamento, $idEntrSaid);

            if(is_null($contapagar[0]['CDTPCTPAGAR']))
                $tipoContNFEntr = $this->lancamentoFactory->getTipoContNF($filial, $lancamento, $idEntrSaid); 
            else
                $tipoContNFEntr = $contapagar[0]['CDTPCTPAGAR'];

            if($tipoContNFEntr) {
                $tipoConta = $tipoContNFEntr['CDTPCTPAGAR'];
                $itemproduto = $this->buscaProdutos($filial, $lancamento);
                
                foreach($itemproduto as $item){
                    $ratCtEntr = $this->lancamentoFactory->getRatCt($tipoConta, $idEntrSaid);
                    foreach($ratCtEntr as $rateioE){
                        if(!is_null($rateioE['CDFILIAL']))
                            $filiRate = $rateioE['CDFILIAL'];
                        else
                            $filiRate = $notaFiscal->getCdfilirece(); //$rateioRow['CDFILIRECE'];

                        $existeRateio = Rateionf::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndCdfiliccustAndCdcentcusnf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, $rateioE['CDCENTCUST']);
                        
                        if(is_null($existeRateio))
                            $this->lancamentoFactory->insertRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioE['VRPERATTPCTP']/100)), $rateioE['CDCENTCUST'], $rateioE['VRPERATTPCTP']);
                        else
                            $this->lancamentoFactory->updateRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioE['VRPERATTPCTP']/100)), $rateioE['CDCENTCUST'], $rateioE['VRPERATTPCTP']);
                    }
                }
            } 
        }
        else { //$idconsctbrat['IDCONSCTBRAT'] <> S
            $this->lancamentoFactory->insertRateionfCompleto($filial, $lancamento, $notaFiscal->getCdfilirece(), $notaFiscal->getIdentrsaidop());
        }
        
    }
    
    public function geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao){
        if($consContaRateio['IDCONSCTBRAT'] == 'S'){
            $tipoContNFSaid = $this->lancamentoFactory->getTipoContNF($filial, $lancamento, $idEntrSaid);
            $tipoConta = $tipoContNFSaid['CDTPCTRECEB'];
            $itemproduto = $this->buscaProdutos($filial, $lancamento);
            
            foreach($itemproduto as $item){
                $ratCtSaid = $this->lancamentoFactory->getRatCt($tipoConta, $idEntrSaid);
                foreach($ratCtSaid as $rateioS){
                    if(!is_null($rateioS['CDFILIAL']))
                        $filiRate = $rateioS['CDFILIAL'];
                    else
                        $filiRate = $notaFiscal->getCdfiliemit(); //$rateio['CDFILIEMIT'];
                    
                    $existeRateio = Rateionf::findOneByCdfilialAndNrlanctonfAndNrseqitprodAndCdfiliccustAndCdcentcusnf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, $rateioS['CDCENTCUST']);
                    if(is_null($existeRateio))
                        $this->lancamentoFactory->insertRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioS['VRPERATTPCTR']/100)), $rateioS['CDCENTCUST'], $rateioS['VRPERATTPCTR']);
                     else
                        $this->lancamentoFactory->updateRateionf($filial, $lancamento, $item['NRSEQITPROD'], $filiRate, ($item['VRCONTITEM'] * ($rateioS['VRPERATTPCTR']/100)), $rateioS['CDCENTCUST'], $rateioS['VRPERATTPCTR']);
                }
            }
        }else{// $idconsctbrat['IDCONSCTBRAT'] <> 'S'
            $this->lancamentoFactory->insertRateionfCompleto($filial, $lancamento, $notaFiscal->getCdfiliemit(), $notaFiscal->getIdentrsaidop());
        }
    }
    
    public function geraRateio($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut = 'N', $realizaManut = 'P'){
        $filiRate = '';
        $organizacao = $this->environment->getNrOrgTrab();
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            //Calcula rateio padrão
            if ($realizaManut != 'S'){
                if($realizaManut == 'P'){
                    $rateionf = Rateionf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                    if(!isset($rateionf)){
                        return;
                    }
                    $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                    $consContaRateio = $this->lancamentoFactory->getConsContaRateio($idEntrSaid, $notaFiscal->getCdtipooper())[0];
                    if($idEntrSaid == 'E'){
                        $this->geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                    }else{ //($idEntrSaid == 'S' SAIDA
                        $this->geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                    }
                }
                else{// $realizaManut == 'S' Calcula rateio padrão
                    if($geraRatAut == 'N' && $realizaManut !== 'P'){ //aqui se rateio manual
                        $rateionf = Rateionf::findByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                        foreach($rateionf as $rateio){
                            $rateio->delete();
                        }
                        $this->entityManager->flush();
                        $notaFiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($filial, $lancamento, $organizacao);
                        $consContaRateio = $this->lancamentoFactory->getConsContaRateio($idEntrSaid, $notaFiscal->getCdtipooper())[0];
                        if($idEntrSaid == 'E'){
                            $this->geraRateioEntrada($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                        }else{ //IDENTRSAID == 'S'
                            $this->geraRateioSaida($filial, $lancamento, $idEntrSaid,  $rateioNF, $geraRatAut, $realizaManut, $consContaRateio, $notaFiscal, $organizacao);
                        }
                    }else{
                        $this->geraRateio($filial, $lancamento, $idEntrSaid, $rateioNF, 'N', 'P');
                        $this->calculaRateioPos($filial, $lancamento, $rateioNF, $organizacao);
                    }
                }
            }
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function calculaRateioPos($filial, $lancamento, $rateioNF , $organizacao){
        if(count($rateioNF) > 0){
            $itemproduto = $this->buscaProdutos($filial, $lancamento);
            foreach($itemproduto as $item){
                foreach($rateioNF as $rateioRow){
                    if($item['NRSEQITPROD'] == $rateioRow['NRSEQITPROD']){
                        $rateionf = Rateionf::findOneByCdfilialAndNrlanctonfAndNrorgAndNrseqitprod($filial, $lancamento, $organizacao, $item['NRSEQITPROD']);
                        $rateionf->delete();
                        $this->entityManager->flush();
                        $this->lancamentoFactory->InsertRatNovProd($rateioRow['CDFILIAL'], $rateioRow['NRLANCTONF'], $rateioRow['NRSEQITPROD'], $rateioRow['CDFILICCUST'], $rateioRow['CDCENTCUSNF'], $rateioRow['VRPERRAT'], $item['VRCONTITEM']);
                    }
                }
            }
            $totalContItem = $this->lancamentoFactory->getTotalCont($lancamento, $filial);
            foreach($totalContItem as $totalItem){
                $somaRateioItem = $this->lancamentoFactory->getSumRateio($lancamento, $filial, $totalItem['NRSEQITPROD'])[0]['VRRATEIONF'];
                if($somaRateioItem > 0){
                    $difCont = $totalItem['VRCONTITEM'] - $somaRateioItem;
                    if((abs($difCont) < 0.10) && (abs($difCont) > 0)){
                        $this->lancamentoFactory->updateRateionfRound($difCont, $filial, $lancamento, $totalItem['NRSEQITPROD']);
                    }
                }
            }
        }
    }
    
    public function verificaDevolucaoOl($filial, $lancamento) {
        
        $itensOl = $this->lancamentoFactory->buscaNotaOl($filial, $lancamento);
        if(isset($itensOl[0])) {
            foreach($itensOl as $item) {
                $this->lancamentoFactory->updateNotaOl($item["CDFILIALIT"], $item["NRLANCTONFIT"], $item["CDPRODUTO"]);
            } 
        }
        
    }
    
    public function salvaCFOPeCST($cdfilial, $nrlanctonf, $nrseqitpro, $cdfisopeitem, $cdsitutribpr) {
        $sql = "UPDATE ITEMPRODUTO
                SET CDFISOPEITEM = :P_CDFISOPEITEM,
                    CDSITUTRIBPR = :P_CDSITUTRIBPR
                WHERE CDFILIAL = :P_CDFILIAL
                  AND NRLANCTONF = :P_NRLANCTONF
                  AND NRSEQITPROD = :P_NRSEQITPROD";
                   
        $params = array("P_CDFISOPEITEM" => $cdfisopeitem,
                        "P_CDSITUTRIBPR" => $cdsitutribpr,
                        "P_CDFILIAL" => $cdfilial,
                        "P_NRLANCTONF" => $nrlanctonf,
                        "P_NRSEQITPROD" => $nrseqitpro);           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getSegMercFilial($cdFilial) {
        $sql = "SELECT
                   F.CDFILIAL, F.CDSEGMER
                FROM
                   FILIAL F
                WHERE
                   F.CDFILIAL = :P_CDFILIAL";
                   
        $params = array("P_CDFILIAL" => $cdFilial);           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }   
    
    public function getErrosAvisos() {
        return $this->errosAvisos;
    }
    
    
    public function getFranqueadora() {
  
        $sql = "SELECT CDFILIAL FROM FILIAL WHERE IDFUNCAOFILI = 'F'";
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        try {
            $std->execute();
            $existe = $std->fetch();
            if(isset($existe["CDFILIAL"])) {
             return $existe["CDFILIAL"];   
            }
            else return false;
        }
        catch(\Exception $e) {
            return false;    
        }

    }
    
    
    public function getDefaultNovaNota($cdFilial, $idEntrSaid) {
        $cdFilial = $cdFilial ? $cdFilial : $this->environment->getCdfilial();
        $cdOperador = $this->environment->getCdoperador();
        $default = $this->lancamentoFactory->getDefaultNovaNota($cdFilial, $cdOperador, $idEntrSaid);
        if(!empty($default)) {
            if($default["DTPROCESSA"]) {
                $default["DTENTRSAID"] = substr($default["DTPROCESSA"], 0, 10);
                if($idEntrSaid == 'E') {
                    $default["DTEMISSAO"]  = substr($default["DTPROCESSA"], 0, 10);
                }
            }
            else {
                $default["DTENTRSAID"] = date("d/m/Y");
                if($idEntrSaid == 'E') {
                    $default["DTEMISSAO"]  = date("d/m/Y");
                }
            }
            $default["CDFILIAL"] = $this->environment->getCdfilial();
        }
        else {
            $default = array("CDFILIAL" => $this->environment->getCdfilial());
        }
        $default["CDFILIAL"] = $this->environment->getCdFilial();
        return $default;
    }
    
    public function getItensNFOrigem($cdFilial, $nrlanctonf){
        $sql = " SELECT
                  A.CDPRODUTO,
                  QTITEMPROD,
                  CDPRODMOVI,
                  CDPRODEMIS,
                  IDCONFORITEM,
                  A.VRTOTAITEM,
                  A.VRCONTITEM,
                  A.VRBRUTITEM,
                  A.VRDESPERITEM,
                  A.CDFILIAL,
                  A.NRLANCTONF,
                  A.NRSEQITPROD,
                  DSPRODEMIS,
                  CDFISOPEITEM,
                  IDATUVRESTOQ,
                  CDSUFXCFOPPR,
                  CDSITUTRIBPR,
                  IDAVISOSITEM,
                  VRUNITITEM,
                  CDCSTINFUSU,
                  CDCFOPINFUSU,
                  CDSUFXINFUSU,
                  CDFILIMOVI,
                  CDALMOXARIFE,
                  CDLOCALESTOQ,
                  NRLOTEESTQ,
                  NRSUBLOTE,
                  VRDESCITEM,
                  CDFILIALPED,
                  NRPEDIDO,
                  NRITEMPED,
                  A.QTVOLUITEM,
                  B.CDARVPROD,
                  A.CDMARCA,
                  A.IDORIGEMPROD,
                  A.IDATUESTPROD
              FROM
                  ITEMPRODUTO   A,
                  PRODUTO       B
              WHERE
                  ( A.CDFILIAL = :P_CDFILIAL )
                  AND ( A.NRLANCTONF = :P_NRLANCTONF )
                  AND ( A.CDPRODUTO = B.CDPRODUTO )
              ORDER BY
                  A.NRSEQITPROD ";
                   
        $params = array("P_CDFILIAL" => $cdFilial,
                        "P_NRLANCTONF" => $nrlanctonf
        );           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }
    
    public function getDadosNF($cdFilial, $nrlanctonf) {
        $sql = "SELECT IDENTRSAIDOP, CDTIPOOPER
                    FROM NOTAFISCAL
                    WHERE CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF";
                   
        $params = array("P_CDFILIAL" => $cdFilial,
                        "P_NRLANCTONF" => $nrlanctonf
        );           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }
    
    public function impostoProduto($cdfilial, $nrlanctonf,$cdimposto) {
        $sql = "
        SELECT A.CDPRODUTO, P.NMPRODUTO,
            A.CDFILIAL, P.CDARVPROD
        FROM ITEMPRODUTO A, PRODUTO P
        WHERE A.CDFILIAL   = :P_CDFILIAL
            AND A.NRLANCTONF = :P_NRLANCTONF
            AND A.CDPRODUTO  = P.CDPRODUTO
            AND NOT EXISTS (
                SELECT CDFILIAL
                FROM ITEMPROIMPOS I
                WHERE I.CDFILIAL = A.CDFILIAL
                    AND I.NRLANCTONF = A.NRLANCTONF
                    AND I.NRSEQITPROD = A.NRSEQITPROD
                    AND I.CDIMPOSTO = :P_CDIMPOSTO
            )
        ";
                   
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf,
            "P_CDIMPOSTO" => $cdimposto
        );           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }
    
    public function impostoObrig($identrsaidop , $cdtipooper) {
        $sql = "SELECT I.CDIMPOSTO, IM.IDPRODNOTA, IM.SGIMPOSTO
                FROM IMPOBRIGTPOPER I, IMPOSTO IM
                WHERE I.IDENTRSAIDOP = :P_IDENTRSAIDOP
                    AND I.CDTIPOOPER = :P_CDTIPOOPER
                    AND IM.CDIMPOSTO = I.CDIMPOSTO";
                   
        $params = array("P_CDTIPOOPER" => $cdtipooper,
                        "P_IDENTRSAIDOP" => $identrsaidop
        );           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }
    
    public function impostoCorpo($cdfilial, $nrlanctonf,$cdimposto) {
        $sql = "SELECT MAX(CDFILIAL) AS CDFILIAL
                FROM IMPOSTONF
                WHERE CDFILIAL   = :P_CDFILIAL
                 AND NRLANCTONF = :P_NRLANCTONF
                 AND CDIMPOSTO  = :P_CDIMPOSTO";
                   
        $params = array("P_CDFILIAL" => $cdfilial,
                        "P_NRLANCTONF" => $nrlanctonf,
                        "P_CDIMPOSTO"=> $cdimposto
        );           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }  
    
    public function deleteImpostoNF($cdfilial, $nrlanctonf, $cdimposto) {
        return $this->lancamentoFactory->deleteImpostoNF($cdfilial, $nrlanctonf, $cdimposto);
    }
    
    public function deleteImpostoItem($cdfilial, $nrlanctonf, $nrorg, $nritproimpos, $nrseqitprod, $cdimposto) {
        return $this->lancamentoFactory->deleteImpostoItem($cdfilial, $nrlanctonf, $nrorg, $nritproimpos, $nrseqitprod, $cdimposto); 
    }
    
    public function insereImpostoItem($params) {
        return $this->lancamentoFactory->insereImpostoItem($params);
		return $std->fetchAll();        
    }
    
    public function insereItemprodutoNFDevolucao($filial, $lancamento, $nrorg) {
        return $this->lancamentoFactory->insereItemprodutoNFDevolucao($filial, $lancamento, $nrorg);
    }
    
    public function getDataForUnidade($CDFILIAL, $NRORG) {
        #requests unidade tab data
        $factoryReturn = $this->lancamentoFactory->getDataForUnidade($CDFILIAL, $NRORG);

        #checks if something was found, throws exception otherwise
        if(count($factoryReturn) != 1)
            throw new \Exception("Parametros da unidade $CDFILIAL não encontrados.", 1);

        #builds data
        $data = $this->buildData($factoryReturn, true);

        #returns data
        return $data;
    }
    
    public function getDataForGeral($NRORG) {
        #requests unidade tab data
        $factoryReturn = $this->lancamentoFactory->getDataForGeral($NRORG);

        #checks if something was found, throws exception otherwise
        if(count($factoryReturn) != 1){
            $this->lancamentoFactory->newParamestoque();
            $factoryReturn = $this->factory->getDataForGeral($NRORG);
            //throw new \Exception("Erro Catastrófico! Erro na requisição dos parâmetros gerais!", 1);
        }

        #builds data
        $data = $this->buildData($factoryReturn, true);

        #returns data
        return $data;
    }
    
    public function buildData($factoryData, $singleRow = false) {
        $data = array();
        if($singleRow) {
            if(count($factoryData))
                foreach($factoryData[0] as $column => $value) {
                    $data[$column] = $value;
                }
        } else {
            foreach($factoryData as $factoryRow) {
                $row = array();
                foreach($factoryRow as $column => $value) {
                    $row[$column] = $value;
                }
                $data[] = $row;
            }
        }

        return $data;
    }
    
    public function novoMultiItemProduto($itens, $cabecalho, &$impostosTemp) {
        foreach ($itens as $item) {
            $filial         = $item['CDFILIAL'];
            $nrlanctonf     = $item['NRLANCTONF'];
            $seqItem        = $item['NRSEQITPROD'];
            $produto        = $item['CDPRODUTO'];
            $codEmissao     = $item['CDPRODEMIS'];
            $descEmissao    = $item['DSPRODEMIS'];
            $quantidade     = $item['QTITEMPROD'];
            $valorUnitario  = $item['VRUNITITEM'];
            $valorDesconto  = $item['VRDESCITEM'];
            $almoxarifado   = array_key_exists("CDALMOXARIFE", $item) ? $item["CDALMOXARIFE"] : null;
            $localEstoque   = array_key_exists("CDLOCALESTOQ", $item) ? $item["CDLOCALESTOQ"] : null;
            $nrloteestq     = array_key_exists("NRLOTEESTQ", $item) ? $item['NRLOTEESTQ'] : null;
            $nrsublote      = array_key_exists("NRSUBLOTE", $item) ? $item['NRSUBLOTE'] : null;
            
            $this->procedureGeraNotaFiscalProduto(
                
                $filial, $nrlanctonf, $produto, $codEmissao, $descEmissao, $quantidade, $valorUnitario, $valorDesconto,
                $almoxarifado, $localEstoque, $nrloteestq, $nrsublote
            );
        }
    }

    public function salvaMultiItensLancamentoSaida($novosItens, $itensAlterados, $cabecalho) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $impostosTemp = array();
            
            //var_dump($novosItens, $itensAlterados, $cabecalho);
            $this->novoMultiItemProduto($novosItens, $cabecalho, $impostosTemp);

            $this->entityManager->flush();
            $connection->commit();

            return $impostosTemp;
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }
    
    public function getInfoTipoOper($cdtipooper) {
        $sql = "SELECT IDOBRDTREFFAT, IDOBRDTREFFATQDCONREC, IDUTILVENC, IDALTDTREFAPROV, IDGERAVENC FROM TIPOOPERAC WHERE CDTIPOOPER = :P_CDTIPOOPER AND IDENTRSAIDOP = 'S'";
                   
        $params = array("P_CDTIPOOPER" => $cdtipooper);           
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();        
    }
    
    public function getStatusNota($cdfilial, $nrlanctonf, $nrorg) {
        $status = $this->lancamentoFactory->getStatusNota($cdfilial, $nrlanctonf, $nrorg);
        return  $status;
    }
    
    /*---------------------------------- Início - Importação de Requisição por Setor ----------------------------------*/
    public function getRequisicoesPorSetor($CDFILIAL, $CDSETOR){
        $requisicoes = $this->lancamentoFactory->getRequisicoesPorSetor($CDFILIAL, $CDSETOR);
        return $requisicoes;
    }
    
    public function getItensPorRequisicaoDeSetor($CDFILIAL, $P_NRREQUESTO){
        $itens = $this->lancamentoFactory->getItensPorRequisicaoDeSetor($CDFILIAL, $P_NRREQUESTO);
        return $itens;
    }
    
    public function getSetorRequisicao(){
        $setores = $this->lancamentoFactory->getSetorRequisicao();
        return $setores;
    }
    
    /*---------------------------------- Final - Importação de Requisição por Setor ----------------------------------*/
    
    /*---------------------------------- Início - Conversão de importação para NF ----------------------------------*/
    public function getVrItens_Requisicao($row){
        try{
            
            $itens = $row["ITENS"];
            $rowAcl = [];
            $itensAcl = [];
            
            $paramFilial = $this->lancamentoFactory->get_paramfilial($row["CDFILIEMIT"]);
            $adqFilialCDSEGMER = $this->getSegMercFilial($row["CDFILIEMIT"]);
            
            //CDTIPOCUSTO
            $row = $this->get_cdtipocusto_by_filial($row);
            
            array_push($rowAcl, $row);
            
            //dtprocessamento/sysdate
            if(!!$paramFilial["DTPROCESSA"]){
                $row["DTENTRSAID"] = $paramFilial["DTPROCESSA"];
            }else{
                $row["DTENTRSAID"] = DateUtil::getDataAtual()->format('d/m/Y');
            }
            
            //search almox/local/lote about product and checks precoBase table or posEstDia;
            foreach($itens as $item){
                
                $prodFili = $this->lancamentoFactory->getParamFilialProdFili($row["CDFILIEMIT"], $item["CDPRODUTO"]);
                $item = $this->get_Almox_Local($paramFilial, $prodFili, $item);
                $item = $this->get_Lote_($prodFili, $row, $item);
                
                $value = $this->validacoesEstoqueService->consultaCustoQuantidadeProduto(
                        $row["CDFILIEMIT"], $item["CDPRODUTO"], $row["DTENTRSAID"], $row["CDTIPOCUSTO"], 
                        $item["CDALMOXARIFE"], $item["CDLOCALESTOQ"], $item["NRLOTEESTQ"], $item["NRSUBLOTE"], $row["CDTIPOOPER"]
                    );
                    
                $item["QTESTOQDIA"] = $value["QTESTOQDIA"];
                $item["VRUNITITEM"] = $value["VRCUSTOPROD"];
                
                if($item["VRUNITITEM"] == 0 || empty($item["VRUNITITEM"])) {
                    $this->erros[] = ("O produto ".$item["CDPRODUTO"]." não tem valor unitário parametrizado. Número da requisição: ".$item["NRREQUESTO"]);
                }
                
                $item["VRTOTAITEM"] = $item["VRUNITITEM"] * $item["QTREQUESTO"];
                
				if($row["tipoRequisicao"] == 'setor') $this->validacoesProdutoSetor($row, $item, $paramFilial, $adqFilialCDSEGMER);      
				
                array_push($itensAcl, $item);
            }
        
        }catch(\Exception $e){
            $itensAcl = [];
            $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
        }
        
        return array("ROW" => $rowAcl, "ITENS" => $itensAcl, "ERROS" => $this->erros);
    }
    
	private function validacoesProdutoSetor($row, $item, $paramFilial, $adqFilialCDSEGMER){
	    
	    $adqFilialCDSEGMER = $adqFilialCDSEGMER[0]["CDSEGMER"];
        $nivel_produto = $this->lancamentoFactory->get_levelProducts($item["CDPRODUTO"]);
        
        //inventario pendente - • O processo não poderá ser realizado quando a unidade de origem tiver com o inventário pendente.
        $inconsistenciaInvPen = $this->lancamentoFactory->verificaInventarioPendente($row["CDFILIEMIT"],$item["CDALMOXARIFE"], $item["CDLOCALESTOQ"], $row["DTENTRSAID"]);
        if(!!$inconsistenciaInvPen)
            $this->erros[] = ('O processo não poderá ser realizado enquanto unidade de origem estiver com o inventário pendente. ');
        
        
        //O produto não poderá possuir composição e deverá estar cadastrado para o módulo filial.
        //O produto deverá estar cadastrado e ativo para o segmento de mercado das unidades
        if ($paramFilial['IDBLPRCOMEST'] == 'S') {
            //Se Bloqueia Produtos com Composição em Compras
            
            if(empty($adqFilialCDSEGMER)){
                //Se não utiliza segmento de mercado
                
                if(!!$nivel_produto){
                    //Se é um produto de nível completo
                    // produto não pode possuir composicao
                    
                    if ($paramFilial['IDUTMODPRODFIL'] == 'N'){ 
                        //Se não utiliza modulo da filial
                        
                        $adqVdProdComp = $this->lancamentoFactory->get_adqVdProdComp($row["CDFILIEMIT"], $item["CDPRODUTO"]);
                        if(empty($adqVdProdComp)){
                            $this->erros[] = ('Produto não pode possuir composição. ' . $item["CDPRODUTO"]);
                        }
                        
                    }else{
                        
                        //Produto não cadastrado para o módulo da filial
                        $adqVdProdComp_Mod = $this->lancamentoFactory->get_adqVdProdComp_Mod($row["CDFILIEMIT"], $item["CDPRODUTO"]);
                        if(empty($adqVdProdComp_Mod)){
                            $this->erros[] = ('Produto não está cadastrado para o módulo da filial. ' . $item["CDPRODUTO"]);
                        }
                    }
                }else{
                    //Se não é um produto de nível completo
                    // produto não pode possuir composicao
                    
                    $adqVdIncomplProd_Comp = $this->lancamentoFactory->get_adqVdIncomplProd_Comp($row["CDFILIEMIT"], $item["CDPRODUTO"]);
                    if(empty($adqVdIncomplProd_Comp)){
                        $this->erros[] = ('Produto não pode possuir composição. ' . $item["CDPRODUTO"]);
                    }
                }
            }else{
                //Se utiliza segmento de mercado
                
                if(!$nivel_produto){
                    //Se é um produto de nível completo
                
                    if ($paramFilial['IDUTMODPRODFIL'] == 'N'){
                        //Se não utiliza modulo da filial
                        //produto não pode possuir composicao
                        
                        $adqVdProdCompSeg = $this->lancamentoFactory->get_adqVdProdCompSeg($row["CDFILIEMIT"], $item["CDPRODUTO"],$adqFilialCDSEGMER);
                        
                        
                        if(empty($adqVdProdCompSeg)){
                            $this->erros[] = ('Produto não pode possuir composição. ' . $item["CDPRODUTO"]);
                            
                        }else{
                            //Produto inativo para o segmento de mercado da filial
                            
                            if($adqVdProdCompSeg['IDPROSEGATIV'] == 'N'){
                                //Produto inativo para o segmento de mercado da filial
                                $this->erros[] = ('Produto inativo para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                            }
                        }
                    }else{
                        //Produto não cadastrado para o módulo da filial
                        
                        $adqVdProdCompSeg_Mod = $this->lancamentoFactory->get_adqVdProdCompSeg_Mod($row["CDFILIEMIT"], $item["CDPRODUTO"],$adqFilialCDSEGMER);
                        
                        if(empty($adqVdProdCompSeg_Mod)){
                            $this->erros[] = ('Produto não cadastrado para o módulo da filial. ' . $item["CDPRODUTO"]);
                        }else{
                            if($adqVdProdCompSeg_Mod['IDPROSEGATIV'] == 'N'){
                                //Produto inativo para o segmento de mercado da filial
                                $this->erros[] = ('Produto inativo para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                            }
                        }
                        
                    }
                    
                }else{
                    //Se não é um produto de nível completo
                    // produto não pode possuir composicao adqVdIncompl_ProdCompSeg
                    
                    $adqVdIncompl_ProdCompSeg = $this->lancamentoFactory->get_adqVdIncompl_ProdCompSeg($row["CDFILIEMIT"], $item["CDPRODUTO"],$adqFilialCDSEGMER);
                
                    if(empty($adqVdIncompl_ProdCompSeg)){
                        $this->erros[] = ('Produto não pode possuir composição. ' . $item["CDPRODUTO"]);
                    }
                    
                }
            }
        }else{
            
            //Se não Bloqueia Produtos com Composição em Compras
            
            if(!empty($adqFilialCDSEGMER)){
                
                //Se utiliza segmento de mercado
                if(!!$nivel_produto){
                    //Se é um produto de nível completo
                    
                    if ($paramFilial['IDUTMODPRODFIL'] == 'N'){
                        //Se não utiliza modulo da filial
                        //Produto não cadastrado para o segmento de mercado da filial.adqVdProdutoSeg
                        
                        $adqVdProdutoSeg = $this->lancamentoFactory->get_adqVdProdutoSeg($item["CDPRODUTO"],$adqFilialCDSEGMER);
                        
                        if(empty($adqVdProdutoSeg)){
                            $this->erros[] = ('Produto não cadastrado para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                        }else{
                            if($adqVdProdCompSeg['IDPROSEGATIV'] == 'N'){
                                //Produto inativo para o segmento de mercado da filial
                                $this->erros[] = ('Produto inativo para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                            }
                        }
                        
                    }else{
                        //Se utiliza modulo da filial
                        //Produto não cadastrado para o segmento de mercado e/ou módulo da filial. adqVdProdutoSeg_Mod
                        
                        $adqVdProdutoSeg_Mod = $this->lancamentoFactory->get_adqVdProdutoSeg_Mod($row["CDFILIEMIT"], $item["CDPRODUTO"], $adqFilialCDSEGMER);
                        
                        if(empty($adqVdProdutoSeg_Mod)){
                            $this->erros[] = ('Produto não cadastrado para o segmento de mercado e/ou módulo da filial. ' . $item["CDPRODUTO"]);
                        }else{
                            if($adqVdProdutoSeg_Mod['IDPROSEGATIV'] == 'N'){
                                //Produto inativo para o segmento de mercado da filial
                                $this->erros[] = ('Produto inativo para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                            }
                        }
                    }
                }else{
                    //Se não é um produto de nível completo
                    //Produto não cadastrado para o segmento de mercado da filial. adqVdIncompl_ProdSeg
                    
                    $adqVdIncompl_ProdSeg = $this->lancamentoFactory->get_adqVdIncompl_ProdSeg($item["CDPRODUTO"], $adqFilialCDSEGMER);
                    
                    if(empty($adqVdIncompl_ProdSeg)){
                        $this->erros[] = ('Produto não cadastrado para o segmento de mercado da filial. ' . $item["CDPRODUTO"]);
                    }
                }
            }else{
                //Se não utiliza segmento de mercado
                
                if(!!$nivel_produto){
                    //Se é um produto de nível completo
                    
                    if ($paramFilial['IDUTMODPRODFIL'] == 'S'){
                        // Se não utiliza modulo da filial
                        //Produto não cadastrado para o módulo da filial.
                        $adqVdProd_Mod = $this->lancamentoFactory->get_adqVdProd_Mod($row["CDFILIEMIT"], $item["CDPRODUTO"]);
                        
                        if(empty($adqVdProd_Mod)){
                            $this->erros[] = ('Produto não cadastrado para o módulo da filial' . $item["CDPRODUTO"]);
                        }
                    }
                }
            }
        }
    }

    private function get_cdtipocusto_by_filial($row){
        $tipooperac = $this->lancamentoFactory->getCDTIPOCUSTO_tipooperac_tipocusto($row["CDTIPOOPER"],$row["IDENTRSAIDOP"]);
        
        if(isset($tipooperac['CDTIPOCUSTO']) && $tipooperac['CDTIPOCUSTO'] != " "){
            $row["CDTIPOCUSTO"] = $tipooperac['CDTIPOCUSTO'];
            
        }else{
            $tipooperac = $this->lancamentoFactory->getCDTIPOCUSTO_paramfilial($row["CDFILIEMIT"]);
            if(isset($tipooperac["IDTCUSNFROMA"]) && $tipooperac["IDTCUSNFROMA"] != " "){
                $row["CDTIPOCUSTO"] = $tipooperac["IDTCUSNFROMA"];
            }else{
                $row["CDTIPOCUSTO"] = "03";
            }
        }
        
        return $row;
    }
    
    private function get_Lote_($prodFili, $row, $item){
        
        $lotSub = $this->procedureGetLote(
                                            $prodFili["IDUTILLOTE"],
                                            $row["CDFILIEMIT"],
                                            $item["CDALMOXARIFE"],
                                            $item["CDLOCALESTOQ"],
                                            $item["CDPRODUTO"]
                                        );

       $lotSub["NRLOTEESTQ"] = $lotSub["P_NRLOTEESTQ"];                                 
       $lotSub["NRSUBLOTE"] = $lotSub["P_NRSUBLOTE"]; 
       
       
        if(isset($lotSub["NRLOTEESTQ"])){
            $item["NRLOTEESTQ"] = $lotSub["NRLOTEESTQ"];
        }else{
            $item["NRLOTEESTQ"] = " ";
        }
        
        if(isset($lotSub["NRSUBLOTE"])){
            $item["NRSUBLOTE"] = $lotSub["NRSUBLOTE"];
        }else{
            $item["NRSUBLOTE"] = " ";
        }
        //var_dump($item);die;  
        return $item;
        
    }
    
    private function get_Almox_Local($paramFilial, $prodFili, $item){
        
        //CDALMOXARIFE
        if($paramFilial["IDCTRLESTQ"] != 'N' && $paramFilial["IDUTILALMOX"] == 'S'){
            if(isset($paramFilial["IDCTRLESTQ"]) && $paramFilial["CDALMOXARIFE"] != ' '){
                $item["CDALMOXARIFE"] = $paramFilial["CDALMOXARIFE"];
            }else{
                if(isset($prodFili["CDALMOXARIFE"]) && $prodFili["CDALMOXARIFE"] != ' '){
                    $item["CDALMOXARIFE"] = $prodFili["CDALMOXARIFE"];
                }else{
                    $item["CDALMOXARIFE"] = " ";
                }
            }
        }else{
            $item["CDALMOXARIFE"] = " ";
        }
        
        //CDLOCALESTOQ
        if($paramFilial["IDCTRLESTQ"] != 'N' && $paramFilial["IDUTILLCESTQ"] == 'S'){
            if(isset($paramFilial["CDLOCALESTOQ"]) && $paramFilial["CDLOCALESTOQ"] != ' '){
                $item["CDLOCALESTOQ"] = $paramFilial["CDLOCALESTOQ"];
            }else{
                if(isset($prodFili["CDLOCALESTOQ"]) && $prodFili["CDLOCALESTOQ"] != ' '){
                    $item["CDLOCALESTOQ"] = $prodFili["CDLOCALESTOQ"];
                }else{
                    $item["CDLOCALESTOQ"] = " ";
                }
            }
        }else{
            $item["CDLOCALESTOQ"] = " ";
        }
        
        return $item;
    }
    /*---------------------------------- Final - Conversão de importação para NF ----------------------------------*/

    
}